@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2011 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2011-2012 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -31,52 +31,15 @@
 #include "mu-contacts.h"
 #include "mu-runtime.h"
 
-enum _OutputFormat {
-	FORMAT_PLAIN,
-	FORMAT_MUTT_ALIAS,
-	FORMAT_MUTT_AB, /* mutt external address book */
-	FORMAT_WL,
-	FORMAT_BBDB,
-	FORMAT_CSV,
-	FORMAT_ORG_CONTACT,
-
-	FORMAT_NONE
-};
-typedef enum _OutputFormat OutputFormat;
-
-static OutputFormat
-get_output_format (const char *formatstr)
-{
-	int i;
-	struct {
-		const char*	name;
-		OutputFormat	format;
-	} formats [] = {
-		{MU_CONFIG_FORMAT_PLAIN,	 FORMAT_PLAIN},
-		{MU_CONFIG_FORMAT_MUTT_ALIAS,    FORMAT_MUTT_ALIAS},
-		{MU_CONFIG_FORMAT_MUTT_AB,       FORMAT_MUTT_AB},
-		{MU_CONFIG_FORMAT_WL,		 FORMAT_WL},
-		{MU_CONFIG_FORMAT_BBDB,		 FORMAT_BBDB},
-		{MU_CONFIG_FORMAT_CSV,	         FORMAT_CSV},
-		{MU_CONFIG_FORMAT_ORG_CONTACT,	 FORMAT_ORG_CONTACT}
-	};
-
-	for (i = 0; i != G_N_ELEMENTS(formats); i++)
-		if (g_strcmp0 (formats[i].name, formatstr) == 0)
-			return formats[i].format;
-
-	return FORMAT_NONE;
-}
-
 static void
-print_header (OutputFormat format)
+print_header (MuConfigFormat format)
 {
 	switch (format) {
-	case FORMAT_BBDB:
-		g_print (";; -*-coding: utf-8-emacs;-*-\n"			 
+	case MU_CONFIG_FORMAT_BBDB:
+		g_print (";; -*-coding: utf-8-emacs;-*-\n"
 			 ";;; file-version: 6\n");
 		break;
-	case FORMAT_MUTT_AB:
+	case MU_CONFIG_FORMAT_MUTT_AB:
 		g_print ("Matching addresses in the mu database:\n");
 		break;
 	default:
@@ -84,8 +47,6 @@ print_header (OutputFormat format)
 	}
 }
 
-
-
 static void
 each_contact_bbdb (const char *email, const char *name, time_t tstamp)
 {
@@ -95,7 +56,7 @@ each_contact_bbdb (const char *email, const char *name, time_t tstamp)
 	lname	  = mu_str_guess_last_name (name);
 	now	  = mu_date_str ("%Y-%m-%d", time(NULL));
 	timestamp = mu_date_str ("%Y-%m-%d", tstamp);
-			
+
 	g_print ("[\"%s\" \"%s\" nil nil nil nil (\"%s\") "
 		 "((creation-date . \"%s\") (time-stamp . \"%s\")) nil]\n",
 		 fname, lname, email, now, timestamp);
@@ -103,35 +64,38 @@ each_contact_bbdb (const char *email, const char *name, time_t tstamp)
 	g_free (now);
 	g_free (timestamp);
 	g_free (fname);
-	g_free (lname);	
+	g_free (lname);
 }
 
 
 static void
 each_contact_mutt_alias (const char *email, const char *name)
 {
-	if (name) {
-		gchar *nick;
+	gchar *nick;
+
+	if (!name)
+		return;
+
+	nick = mu_str_guess_nick (name);
+	mu_util_print_encoded ("alias %s %s <%s>\n",
+			       nick, name, email);
+	g_free (nick);
 
-		nick = mu_str_guess_nick (name);
-		mu_util_print_encoded ("alias %s %s <%s>\n",
-				       nick, name, email);
-		
-		g_free (nick);
-	}
 }
 
 
 static void
 each_contact_wl (const char *email, const char *name)
 {
-	if (name) {
-		gchar *nick;
-		nick = mu_str_guess_nick (name);
-		mu_util_print_encoded ("%s \"%s\" \"%s\"\n",
-				       email, nick, name);
-		g_free (nick);
-	}
+	gchar *nick;
+
+	if (!name)
+		return;
+
+	nick = mu_str_guess_nick (name);
+	mu_util_print_encoded ("%s \"%s\" \"%s\"\n",
+			       email, nick, name);
+	g_free (nick);
 }
 
 
@@ -153,14 +117,19 @@ print_plain (const char *email, const char *name, gboolean color)
 		fputs (" ", stdout);
 	}
 
-	if (color) fputs (MU_COLOR_GREEN, stdout);
+	if (color)
+		fputs (MU_COLOR_GREEN, stdout);
+
 	mu_util_fputs_encoded (email, stdout);
-	if (color) fputs (MU_COLOR_DEFAULT, stdout);
+
+	if (color)
+		fputs (MU_COLOR_DEFAULT, stdout);
+
 	fputs ("\n", stdout);
 }
 
 struct _ECData {
-	OutputFormat format;
+	MuConfigFormat format;
 	gboolean color;
 };
 typedef struct _ECData ECData;
@@ -171,14 +140,23 @@ each_contact (const char *email, const char *name, time_t tstamp,
 	      ECData *ecdata)
 {
 	switch (ecdata->format) {
-	case FORMAT_MUTT_ALIAS: each_contact_mutt_alias (email, name); break;
-	case FORMAT_MUTT_AB:
+	case MU_CONFIG_FORMAT_MUTT_ALIAS:
+		each_contact_mutt_alias (email, name);
+		break;
+	case MU_CONFIG_FORMAT_MUTT_AB:
 		mu_util_print_encoded ("%s\t%s\t\n",
-				       email, name ? name : ""); break;
-	case FORMAT_WL: each_contact_wl (email, name); break;
-	case FORMAT_ORG_CONTACT: each_contact_org_contact (email, name); break;
-	case FORMAT_BBDB: each_contact_bbdb (email, name, tstamp); break;	
-        case FORMAT_CSV:
+				       email, name ? name : "");
+		break;
+	case MU_CONFIG_FORMAT_WL:
+		each_contact_wl (email, name);
+		break;
+	case MU_CONFIG_FORMAT_ORG_CONTACT:
+		each_contact_org_contact (email, name);
+		break;
+	case MU_CONFIG_FORMAT_BBDB:
+		each_contact_bbdb (email, name, tstamp);
+		break;
+        case MU_CONFIG_FORMAT_CSV:
 		mu_util_print_encoded ("%s,%s\n", name ? name : "", email);
 		break;
 	default:
@@ -187,49 +165,52 @@ each_contact (const char *email, const char *name, time_t tstamp,
 }
 
 
-static MuExitCode
-run_cmd_cfind (const char* pattern, OutputFormat format,
-	       gboolean color)
+static MuError
+run_cmd_cfind (const char* pattern, MuConfigFormat format,
+	       gboolean color, GError **err)
 {
 	gboolean rv;
 	MuContacts *contacts;
 	size_t num;
-	ECData ecdata = {format, color};
-	
+	ECData ecdata;
+
+	ecdata.format = format;
+	ecdata.color  = color;
+
 	contacts = mu_contacts_new (mu_runtime_path(MU_RUNTIME_PATH_CONTACTS));
 	if (!contacts) {
-		g_warning ("could not retrieve contacts");
-		return MU_EXITCODE_ERROR;
+		g_set_error (err, 0, MU_ERROR_CONTACTS_CANNOT_RETRIEVE,
+				     "could not retrieve contacts");
+		return MU_ERROR_CONTACTS_CANNOT_RETRIEVE;
 	}
 
 	print_header (format);
 	rv = mu_contacts_foreach (contacts,
 				  (MuContactsForeachFunc)each_contact,
 				  &ecdata, pattern, &num);
-	
 	mu_contacts_destroy (contacts);
 
 	if (num == 0) {
 		g_warning ("no matching contacts found");
-		return MU_EXITCODE_NO_MATCHES;
+		return MU_ERROR_NO_MATCHES;
 	}
 
-	return rv ? MU_EXITCODE_OK : MU_EXITCODE_ERROR;
-
-	
+	return rv ? MU_OK : MU_ERROR_CONTACTS;
 }
 
-MuExitCode
-mu_cmd_cfind (MuConfig *opts)
+static gboolean
+cfind_params_valid (MuConfig *opts)
 {
-	OutputFormat format;
-	
-	g_return_val_if_fail (opts, MU_EXITCODE_ERROR);
-	g_return_val_if_fail (opts->cmd == MU_CONFIG_CMD_CFIND,
-			      MU_EXITCODE_ERROR);
-
-	format = get_output_format (opts->formatstr);
-	if (format == FORMAT_NONE) {
+	switch (opts->format) {
+	case MU_CONFIG_FORMAT_PLAIN:
+	case MU_CONFIG_FORMAT_MUTT_ALIAS:
+	case MU_CONFIG_FORMAT_MUTT_AB:
+	case MU_CONFIG_FORMAT_WL:
+	case MU_CONFIG_FORMAT_BBDB:
+	case MU_CONFIG_FORMAT_CSV:
+	case MU_CONFIG_FORMAT_ORG_CONTACT:
+		break;
+	default:
 		g_warning ("invalid output format %s",
 			   opts->formatstr ? opts->formatstr : "<none>");
 		return FALSE;
@@ -238,9 +219,26 @@ mu_cmd_cfind (MuConfig *opts)
 	/* only one pattern allowed */
 	if (opts->params[1] && opts->params[2]) {
 		g_warning ("usage: mu cfind [options] [<ptrn>]");
-		return MU_EXITCODE_ERROR;
+		return FALSE;
 	}
 
-	return run_cmd_cfind (opts->params[1], format, opts->color);
+	return TRUE;
 }
 
+
+MuError
+mu_cmd_cfind (MuConfig *opts, GError **err)
+{
+	g_return_val_if_fail (opts, MU_ERROR_INTERNAL);
+	g_return_val_if_fail (opts->cmd == MU_CONFIG_CMD_CFIND,
+			      MU_ERROR_INTERNAL);
+
+	if (!cfind_params_valid (opts)) {
+		g_set_error (err, 0, MU_ERROR_IN_PARAMETERS,
+			     "invalid parameters");
+		return MU_ERROR_IN_PARAMETERS;
+	}
+
+	return run_cmd_cfind (opts->params[1], opts->format,
+			      !opts->nocolor, err);
+}
