@@ -1,7 +1,7 @@
 /* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
 
 /*
-** Copyright (C) 2010-2011 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2010-2012 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -38,14 +38,20 @@ save_part (MuMsg *msg, const char *targetdir, guint partidx, gboolean overwrite,
 {
 	GError *err;
 	gchar *filepath;
-	
-	filepath = mu_msg_part_filepath (msg, targetdir, partidx);
+
+	err = NULL;
+
+	filepath = mu_msg_part_filepath (msg, targetdir, partidx, &err);
 	if (!filepath) {
-		g_warning ("failed to get filepath");
+		if (err) {
+			g_warning ("failed to save MIME-part: %s",
+				   err->message);
+			g_error_free (err);
+		}
+		g_free (filepath);
 		return FALSE;
 	}
-	
-	err = NULL;
+
 	if (!mu_msg_part_save (msg, filepath, partidx, overwrite, FALSE, &err)) {
 		if (err) {
 			g_warning ("failed to save MIME-part: %s",
@@ -59,7 +65,7 @@ save_part (MuMsg *msg, const char *targetdir, guint partidx, gboolean overwrite,
 	if (play)
 		mu_util_play (filepath, TRUE, FALSE);
 
-	g_free (filepath);	
+	g_free (filepath);
 	return TRUE;
 }
 
@@ -70,22 +76,22 @@ save_numbered_parts (MuMsg *msg, MuConfig *opts)
 {
 	gboolean rv;
 	char **parts, **cur;
-	
+
 	parts = g_strsplit (opts->parts, ",", 0);
-	
+
 	for (rv = TRUE, cur = parts; cur && *cur; ++cur) {
 
 		unsigned idx;
 		int i;
 		char *endptr;
-		
+
 		idx = (unsigned)(i = strtol (*cur, &endptr, 10));
 		if (i < 0 || *cur == endptr) {
 			g_warning ("invalid MIME-part index '%s'", *cur);
 			rv = FALSE;
 			break;
 		}
-		
+
 		if (!save_part (msg, opts->targetdir, idx, opts->overwrite,
 				opts->play)) {
 			g_warning ("failed to save MIME-part %d", idx);
@@ -105,7 +111,7 @@ anchored_regex (const char* pattern)
 	GError *err;
 	gchar *anchored;
 
-	
+
 	anchored = g_strdup_printf
 		("%s%s%s",
 		 pattern[0] == '^' ? "" : "^",
@@ -123,7 +129,7 @@ anchored_regex (const char* pattern)
 		g_error_free (err);
 		return NULL;
 	}
-	
+
 	return rx;
 }
 
@@ -139,7 +145,7 @@ save_part_with_filename (MuMsg *msg, const char *pattern, MuConfig *opts)
 	rx = anchored_regex (pattern);
 	if (!rx)
 		return FALSE;
-	
+
 	lst = mu_msg_part_find_files (msg, rx);
 	g_regex_unref (rx);
 	if (!lst) {
@@ -152,7 +158,7 @@ save_part_with_filename (MuMsg *msg, const char *pattern, MuConfig *opts)
 				 GPOINTER_TO_UINT(cur->data),
 				 opts->overwrite, opts->play);
 	g_slist_free (lst);
-	
+
 	return rv;
 }
 
@@ -175,12 +181,12 @@ ignore_part (MuMsg *msg, MuMsgPart *part, SaveData *sd)
 	/* something went wrong somewhere; stop */
 	if (!sd->result)
 		return TRUE;
-	
+
 	/* filter out non-attachments if only want attachments */
 	if (sd->attachments_only &&
 	    !mu_msg_part_looks_like_attachment (part, TRUE))
 		return TRUE;
-	
+
 	/* ignore multiparts */
 	if (part->type &&
 	    g_ascii_strcasecmp (part->type, "multipart") == 0)
@@ -196,28 +202,31 @@ save_part_if (MuMsg *msg, MuMsgPart *part, SaveData *sd)
 	gchar *filepath;
 	gboolean rv;
 	GError *err;
-	
+
 	if (ignore_part (msg, part, sd))
 		return;
 
 	rv	 = FALSE;
 	filepath = NULL;
-	
-	filepath = mu_msg_part_filepath (msg, sd->targetdir, part->index);
-	if (!filepath) 
-		goto leave;
 
 	err = NULL;
+	filepath = mu_msg_part_filepath (msg, sd->targetdir, part->index, &err);
+	if (!filepath) {
+		g_warning ("failed to get file path: %s",
+			   err&&err->message ? err->message : "error");
+		g_clear_error (&err);
+		goto leave;
+	}
+
 	if (!mu_msg_part_save (msg, filepath, part->index,
 			       sd->overwrite, FALSE, &err)) {
 		g_warning ("failed to save MIME-part: %s",
 			   err&&err->message ? err->message : "error");
-		if (err)
-			g_error_free (err);
+		g_clear_error (&err);
 		goto leave;
 	}
-	
-	if (sd->play &&  !mu_util_play (filepath, TRUE, FALSE))
+
+	if (sd->play && !mu_util_play (filepath, TRUE, FALSE))
 		goto leave;
 
 	rv = TRUE;
@@ -240,28 +249,28 @@ save_certain_parts (MuMsg *msg, gboolean attachments_only,
 	sd.overwrite	    = overwrite;
 	sd.targetdir	    = targetdir;
 	sd.play             = play;
-	
-	mu_msg_part_foreach (msg,
+
+	mu_msg_part_foreach (msg, FALSE,
 			     (MuMsgPartForeachFunc)save_part_if,
 			     &sd);
-	
+
 	if (sd.saved_num == 0) {
 		g_warning ("no %s extracted from this message",
 			   attachments_only ? "attachments" : "parts");
 		sd.result = FALSE;
 	}
-	
+
 	return sd.result;
 }
 
 
 static gboolean
 save_parts (const char *path, const char *filename, MuConfig *opts)
-{	
+{
 	MuMsg* msg;
 	gboolean rv;
 	GError *err;
-	
+
 	err = NULL;
 	msg = mu_msg_new_from_file (path, NULL, &err);
 	if (!msg) {
@@ -271,10 +280,10 @@ save_parts (const char *path, const char *filename, MuConfig *opts)
 		}
 		return FALSE;
 	}
-		
+
 	/* note, mu_cmd_extract already checks whether what's in opts
 	 * is somewhat, so no need for extensive checking here */
-	
+
 	/* should we save some explicit parts? */
 	if (opts->parts)
 		rv = save_numbered_parts (msg, opts);
@@ -290,9 +299,9 @@ save_parts (const char *path, const char *filename, MuConfig *opts)
 					 opts->play);
 	else
 		g_assert_not_reached ();
-		
+
 	mu_msg_unref (msg);
-	
+
 	return rv;
 }
 
@@ -319,37 +328,38 @@ each_part_show (MuMsg *msg, MuMsgPart *part, gboolean color)
 	color_maybe (MU_COLOR_MAGENTA);
 	mu_util_print_encoded (
 		"[%s]",	part->disposition ? part->disposition : "<none>");
+
+
+	/* size */
+	if (part->size > 0) {
+		color_maybe (MU_COLOR_CYAN);
+		g_print (" (%s)", mu_str_size_s (part->size));
+	}
+
 	color_maybe (MU_COLOR_DEFAULT);
 	fputs ("\n", stdout);
 }
 
 
 static gboolean
-show_parts (const char* path, MuConfig *opts)
+show_parts (const char* path, MuConfig *opts, GError **err)
 {
 	MuMsg* msg;
-	GError *err;
 
-	err = NULL;
-	msg = mu_msg_new_from_file (path, NULL, &err);
-	if (!msg) {
-		if (err) {
-			g_warning ("error: %s", err->message);
-			g_error_free (err);
-		}
+	msg = mu_msg_new_from_file (path, NULL, err);
+	if (!msg)
 		return FALSE;
-	}
 
 	g_print ("MIME-parts in this message:\n");
 
 	mu_msg_part_foreach
-		(msg,(MuMsgPartForeachFunc)each_part_show,
-		 GUINT_TO_POINTER(opts->color));
+		(msg, FALSE, (MuMsgPartForeachFunc)each_part_show,
+		 GUINT_TO_POINTER(!opts->nocolor));
 
 	mu_msg_unref (msg);
-	
+
 	return TRUE;
-	
+
 }
 
 
@@ -359,19 +369,19 @@ check_params (MuConfig *opts)
 	guint param_num;
 
 	param_num = mu_config_param_num(opts);
-	
+
 	if (param_num < 2) {
 		g_warning ("usage: mu extract [options] <file> [<pattern>]");
 		return FALSE;
 	}
-	
+
 	if (opts->save_attachments || opts->save_all)
 		if (opts->parts || param_num == 3) {
 			g_warning ("--save-attachments --save-all don't accept "
 				   "a filename pattern or --parts");
 			return FALSE;
 		}
-	
+
 	if (opts->save_attachments && opts->save_all) {
 		g_warning ("only one of --save-attachments and"
 			   " --save-all is allowed");
@@ -381,31 +391,35 @@ check_params (MuConfig *opts)
 	return TRUE;
 }
 
-MuExitCode
-mu_cmd_extract (MuConfig *opts)
+MuError
+mu_cmd_extract (MuConfig *opts, GError **err)
 {
 	int rv;
-	
-	g_return_val_if_fail (opts, MU_EXITCODE_ERROR);
+
+	g_return_val_if_fail (opts, MU_ERROR_INTERNAL);
 	g_return_val_if_fail (opts->cmd == MU_CONFIG_CMD_EXTRACT,
-			      MU_EXITCODE_ERROR);
-	
-	if (!check_params (opts))
-		return MU_EXITCODE_ERROR;
-	
+			      MU_ERROR_INTERNAL);
+
+	if (!check_params (opts)) {
+		g_set_error (err, 0, MU_ERROR_IN_PARAMETERS,
+				     "error in parameters");
+		return MU_ERROR_IN_PARAMETERS;
+	}
+
 	if (!opts->params[2] && !opts->parts &&
-	    !opts->save_attachments && !opts->save_all) 
-		rv = show_parts (opts->params[1], opts); /* show, don't save */
+	    !opts->save_attachments && !opts->save_all)
+		rv = show_parts (opts->params[1], opts, err); /* show, don't save */
 	else {
 		rv = mu_util_check_dir(opts->targetdir, FALSE, TRUE);
 		if (!rv)
-			g_warning ("target '%s' is not a writable directory",
-				   opts->targetdir);
+			g_set_error (err, 0, MU_ERROR_FILE_CANNOT_WRITE,
+				     "target '%s' is not a writable directory",
+				     opts->targetdir);
 		else
 			rv = save_parts (opts->params[1],
 					 opts->params[2],
 					 opts); /* save */
 	}
-		
-	return rv ? MU_EXITCODE_OK : MU_EXITCODE_ERROR;
+
+	return rv ? MU_OK : MU_ERROR;
 }
