@@ -1,6 +1,6 @@
 /* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
 
-/* 
+/*
 ** Copyright (C) 2008-2011 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
@@ -15,8 +15,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software Foundation,
-** Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
-**  
+** Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+**
 */
 
 #if HAVE_CONFIG_H
@@ -69,15 +69,6 @@ create_maildir (const char *path, mode_t mode, GError **err)
 	int i;
 	const gchar* subdirs[] = {"new", "cur", "tmp"};
 
-	/* make sure it does not exist yet */
-	if (access (path, F_OK) == 0)
-		errno = EEXIST;
-
-	if (errno != ENOENT) {
-		g_set_error (err, 0, MU_ERROR_FILE, "%s", strerror (errno));
-		return FALSE;
-	}
-	
 	for (i = 0; i != G_N_ELEMENTS(subdirs); ++i) {
 
 		const char *fullpath;
@@ -85,21 +76,32 @@ create_maildir (const char *path, mode_t mode, GError **err)
 
 		/* static buffer */
 		fullpath = mu_str_fullpath_s (path, subdirs[i]);
+
+		/* if subdir already exists, don't try to re-create
+		 * it */
+		if (mu_util_check_dir (fullpath, TRUE, TRUE))
+			continue;
+
 		rv = g_mkdir_with_parents (fullpath, (int)mode);
-		if (rv != 0) {
+
+		/* note, g_mkdir_with_parents won't detect an error if
+		 * there's already such a dir, but with the wrong
+		 * permissions; so we need to check */
+		if (rv != 0 || !mu_util_check_dir(fullpath, TRUE, TRUE)) {
 			g_set_error (err, 0, MU_ERROR_FILE_CANNOT_MKDIR,
-				     "g_mkdir_with_parents failed: %s",
+				     "creating dir failed for %s: %s",
+				     fullpath,
 				     strerror (errno));
 			return FALSE;
 		}
 	}
-	
+
 	return TRUE;
 }
 
 static gboolean
 create_noindex (const char *path, GError **err)
-{	
+{
 	/* create a noindex file if requested */
 	int fd;
 	const char *noindexpath;
@@ -108,7 +110,7 @@ create_noindex (const char *path, GError **err)
 	noindexpath = mu_str_fullpath_s (path, MU_MAILDIR_NOINDEX_FILE);
 
 	fd = creat (noindexpath, 0644);
-	
+
 	/* note, if the 'close' failed, creation may still have
 	 * succeeded...*/
 	if (fd < 0 || close (fd) != 0) {
@@ -117,24 +119,24 @@ create_noindex (const char *path, GError **err)
 			     strerror (errno));
 		return FALSE;
 	}
-	
+
 	return TRUE;
 }
 
 gboolean
 mu_maildir_mkdir (const char* path, mode_t mode, gboolean noindex, GError **err)
-{	
+{
 	g_return_val_if_fail (path, FALSE);
-	
+
 	MU_WRITE_LOG ("%s (%s, %o, %s)", __FUNCTION__,
 		      path, mode, noindex ? "TRUE" : "FALSE");
-	
+
 	if (!create_maildir (path, mode, err))
 		return FALSE;
 
 	if (noindex && !create_noindex (path, err))
 		return FALSE;
-	
+
 	return TRUE;
 }
 
@@ -157,7 +159,7 @@ check_subdir (const char *src, gboolean *in_cur, GError **err)
 		return FALSE;
 	}
 	g_free (srcpath);
-	
+
 	return TRUE;
 }
 
@@ -166,10 +168,10 @@ get_target_fullpath (const char* src, const gchar *targetpath, GError **err)
 {
 	gchar *targetfullpath, *srcfile;
 	gboolean in_cur;
-	
+
 	if (!check_subdir (src, &in_cur, err))
 		return NULL;
-	
+
 	srcfile = g_path_get_basename (src);
 
 	/* create targetpath; note: make the filename cough* unique by
@@ -184,7 +186,7 @@ get_target_fullpath (const char* src, const gchar *targetpath, GError **err)
 					  g_str_hash(src),
 					  srcfile);
 	g_free (srcfile);
-	
+
 	return targetfullpath;
 }
 
@@ -194,16 +196,16 @@ mu_maildir_link (const char* src, const char *targetpath, GError **err)
 {
 	gchar *targetfullpath;
 	int rv;
-	
+
 	g_return_val_if_fail (src, FALSE);
 	g_return_val_if_fail (targetpath, FALSE);
-	
+
 	targetfullpath = get_target_fullpath (src, targetpath, err);
 	if (!targetfullpath)
 		return FALSE;
-	
+
 	rv = symlink (src, targetfullpath);
-	
+
 	if (rv != 0) {
 		g_set_error (err, 0, MU_ERROR_FILE_CANNOT_LINK,
 			     "error creating link %s => %s: %s",
@@ -217,17 +219,18 @@ mu_maildir_link (const char* src, const char *targetpath, GError **err)
 }
 
 
-static MuResult process_dir (const char* path, const gchar *mdir,
-			     MuMaildirWalkMsgCallback msg_cb, 
-			     MuMaildirWalkDirCallback dir_cb, void *data);
+static MuError
+process_dir (const char* path, const gchar *mdir,
+	     MuMaildirWalkMsgCallback msg_cb,
+	     MuMaildirWalkDirCallback dir_cb, void *data);
 
-static MuResult 
+static MuError
 process_file (const char* fullpath, const gchar* mdir,
 	      MuMaildirWalkMsgCallback msg_cb, void *data)
 {
-	MuResult result;
+	MuError result;
 	struct stat statbuf;
-	
+
 	if (!msg_cb)
 		return MU_OK;
 
@@ -236,18 +239,18 @@ process_file (const char* fullpath, const gchar* mdir,
 			   strerror(errno));
 		return MU_ERROR;
 	}
-	
+
 	if (G_UNLIKELY(stat (fullpath, &statbuf) != 0)) {
 		g_warning ("cannot stat %s: %s", fullpath, strerror(errno));
 		return MU_ERROR;
 	}
-	
+
 	result = (msg_cb)(fullpath, mdir, &statbuf, data);
-	if (result == MU_STOP) 
+	if (result == MU_STOP)
 		g_debug ("callback said 'MU_STOP' for %s", fullpath);
 	else if (result == MU_ERROR)
-		g_warning ("%s: failed %d in callback (%s)",  
-			   __FUNCTION__, result, fullpath);
+		g_warning ("%s: error in callback (%s)",
+			   __FUNCTION__, fullpath);
 
 	return result;
 }
@@ -257,29 +260,29 @@ process_file (const char* fullpath, const gchar* mdir,
  * determine if path is a maildir leaf-dir; ie. if it's 'cur' or 'new'
  * (we're skipping 'tmp' for obvious reasons)
  */
-G_GNUC_CONST static gboolean 
+G_GNUC_CONST static gboolean
 is_maildir_new_or_cur (const char *path)
 {
 	size_t len;
 
 	g_return_val_if_fail (path, FALSE);
-	
+
 	/* path is the full path; it cannot possibly be shorter
 	 * than 4 for a maildir (/cur or /new) */
 	len = strlen (path);
 	if (G_UNLIKELY(len < 4))
 		return FALSE;
-	
+
 	/* optimization; one further idea would be cast the 4 bytes to an integer
 	 * and compare that -- need to think about alignment, endianness */
-	
-	if (path[len - 4] == G_DIR_SEPARATOR &&	
+
+	if (path[len - 4] == G_DIR_SEPARATOR &&
 	    path[len - 3] == 'c' &&
 	    path[len - 2] == 'u' &&
 	    path[len - 1] == 'r')
 		return TRUE;
 
-	if (path[len - 4] == G_DIR_SEPARATOR &&	
+	if (path[len - 4] == G_DIR_SEPARATOR &&
 	    path[len - 3] == 'n' &&
 	    path[len - 2] == 'e' &&
 	    path[len - 1] == 'w')
@@ -303,7 +306,7 @@ has_noindex_file (const char *path)
 	else if (G_UNLIKELY(errno != ENOENT))
 		g_warning ("error testing for noindex file %s: %s",
 			   noindexpath, strerror(errno));
-	
+
 	return FALSE;
 }
 
@@ -317,7 +320,7 @@ is_dotdir_to_ignore (const char* dir)
 		".nnmaildir",
 		".#evolution"
 	}; /* when adding names, check the optimization below */
-	
+
 	if (dir[0] != '.')
 		return FALSE; /* not a dotdir */
 
@@ -338,16 +341,12 @@ is_dotdir_to_ignore (const char* dir)
 static gboolean
 ignore_dir_entry (struct dirent *entry, unsigned char d_type)
 {
-	const char *name;
-	
 	/* if it's not a dir and not a file, ignore it.
 	 * note, this means also symlinks (DT_LNK) are ignored,
 	 * maybe make this optional */
 	if (G_UNLIKELY(d_type != DT_REG && d_type != DT_DIR))
 		return TRUE;
-	
-	name = entry->d_name;
-	
+
 	/* ignore '.' and '..' dirs, as well as .notmuch and
 	 * .nnmaildir */
 
@@ -367,16 +366,16 @@ get_mdir_for_path (const gchar *old_mdir, const gchar *dir)
 }
 
 
-static MuResult
+static MuError
 process_dir_entry (const char* path, const char* mdir, struct dirent *entry,
 		   MuMaildirWalkMsgCallback cb_msg,
-		   MuMaildirWalkDirCallback cb_dir, 
+		   MuMaildirWalkDirCallback cb_dir,
 		   void *data)
 {
 	const char *fp;
 	char* fullpath;
 	unsigned char d_type;
-	
+
 	/* we have to copy the buffer from fullpath_s, because it
 	 * returns a static buffer, and we maybe called reentrantly */
 	fp = mu_str_fullpath_s (path, entry->d_name);
@@ -384,21 +383,21 @@ process_dir_entry (const char* path, const char* mdir, struct dirent *entry,
 	strcpy (fullpath, fp);
 
 	d_type = GET_DTYPE(entry, fullpath);
-	
+
 	/* ignore special files/dirs */
-	if (ignore_dir_entry (entry, d_type)) 
+	if (ignore_dir_entry (entry, d_type))
 		return MU_OK;
-	
+
 	switch (d_type) {
 	case DT_REG: /* we only want files in cur/ and new/ */
-		if (!is_maildir_new_or_cur (path)) 
+		if (!is_maildir_new_or_cur (path))
 			return MU_OK;
-		
+
 		return process_file (fullpath, mdir, cb_msg, data);
-		
+
 	case DT_DIR: {
 		char *my_mdir;
-		MuResult rv;
+		MuError rv;
 
 		my_mdir = get_mdir_for_path (mdir, entry->d_name);
 		rv = process_dir (fullpath, my_mdir, cb_msg, cb_dir, data);
@@ -406,14 +405,14 @@ process_dir_entry (const char* path, const char* mdir, struct dirent *entry,
 
 		return rv;
 	}
-		
+
 	default:
 		return MU_OK; /* ignore other types */
 	}
 }
 
 
-static struct dirent* 
+static struct dirent*
 dirent_copy (struct dirent *entry)
 {
 	struct dirent *d;
@@ -431,7 +430,7 @@ dirent_destroy (struct dirent *entry)
 	g_slice_free (struct dirent, entry);
 }
 
-#ifdef HAVE_STRUCT_DIRENT_D_INO	
+#ifdef HAVE_STRUCT_DIRENT_D_INO
 static int
 dirent_cmp (struct dirent *d1, struct dirent *d2)
 {
@@ -447,28 +446,28 @@ dirent_cmp (struct dirent *d1, struct dirent *d2)
 }
 #endif /*HAVE_STRUCT_DIRENT_D_INO*/
 
-static MuResult
+static MuError
 process_dir_entries (DIR *dir, const char* path, const char* mdir,
 		     MuMaildirWalkMsgCallback msg_cb,
 		     MuMaildirWalkDirCallback dir_cb, void *data)
 {
-	MuResult result;
+	MuError result;
 	GSList *lst, *c;
 	struct dirent *entry;
-	
+
 	lst = NULL;
 	while ((entry = readdir (dir)))
 		lst = g_slist_prepend (lst, dirent_copy(entry));
-	
+
 	/* we sort by inode; this makes things much faster on
 	 * extfs2,3 */
-#if HAVE_STRUCT_DIRENT_D_INO		
+#if HAVE_STRUCT_DIRENT_D_INO
 	c = lst = g_slist_sort (lst, (GCompareFunc)dirent_cmp);
-#endif /*HAVE_STRUCT_DIRENT_D_INO*/	
+#endif /*HAVE_STRUCT_DIRENT_D_INO*/
 
 	for (c = lst, result = MU_OK; c && result == MU_OK; c = g_slist_next(c)) {
 		result = process_dir_entry (path, mdir,
-					    (struct dirent*)c->data, 
+					    (struct dirent*)c->data,
 					    msg_cb, dir_cb, data);
 		/* hmmm, break on MU_ERROR as well? */
 		if (result == MU_STOP)
@@ -477,70 +476,70 @@ process_dir_entries (DIR *dir, const char* path, const char* mdir,
 
 	g_slist_foreach (lst, (GFunc)dirent_destroy, NULL);
 	g_slist_free (lst);
-	
+
 	return result;
 }
 
 
-static MuResult
+static MuError
 process_dir (const char* path, const char* mdir,
-	     MuMaildirWalkMsgCallback msg_cb, 
+	     MuMaildirWalkMsgCallback msg_cb,
 	     MuMaildirWalkDirCallback dir_cb, void *data)
 {
-	MuResult result;
+	MuError result;
 	DIR* dir;
-	
+
 	/* if it has a noindex file, we ignore this dir */
 	if (has_noindex_file (path)) {
 		g_debug ("found .noindex: ignoring dir %s", path);
 		return MU_OK;
 	}
 
-	dir = opendir (path);		
+	dir = opendir (path);
 	if (G_UNLIKELY(!dir)) {
 		g_warning ("%s: ignoring  %s: %s",  __FUNCTION__,
 			   path, strerror(errno));
 		return MU_OK;
 	}
-	
+
 	if (dir_cb) {
-		MuResult rv;
+		MuError rv;
 		rv = dir_cb (path, TRUE, data);
 		if (rv != MU_OK) {
 			closedir (dir);
 			return rv;
 		}
 	}
-	
+
 	result = process_dir_entries (dir, path, mdir, msg_cb, dir_cb, data);
 	closedir (dir);
 
 	/* only run dir_cb if it exists and so far, things went ok */
 	if (dir_cb && result == MU_OK)
 		return dir_cb (path, FALSE, data);
-	
+
 	return result;
 }
 
 
-MuResult
-mu_maildir_walk (const char *path, MuMaildirWalkMsgCallback cb_msg, 
+MuError
+mu_maildir_walk (const char *path, MuMaildirWalkMsgCallback cb_msg,
 		 MuMaildirWalkDirCallback cb_dir, void *data)
 {
-	MuResult rv;
+	MuError rv;
 	char *mypath;
-	
+
 	g_return_val_if_fail (path && cb_msg, MU_ERROR);
-	g_return_val_if_fail (mu_util_check_dir(path, TRUE, FALSE), MU_ERROR);	
-	
+	g_return_val_if_fail (mu_util_check_dir(path, TRUE, FALSE), MU_ERROR);
+
 	/* strip the final / or \ */
 	mypath = g_strdup (path);
 	if (mypath[strlen(mypath)-1] == G_DIR_SEPARATOR)
 		mypath[strlen(mypath)-1] = '\0';
-	
+
 	rv = process_dir (mypath, NULL, cb_msg, cb_dir, data);
 	g_free (mypath);
-	
+
 	return rv;
 }
 
@@ -550,32 +549,32 @@ clear_links (const gchar* dirname, DIR *dir, GError **err)
 {
 	struct dirent *entry;
 	gboolean rv;
-	
+
 	rv = TRUE;
 	errno = 0;
 	while ((entry = readdir (dir))) {
-		
+
 		const char *fp;
 		char *fullpath;
 		unsigned char d_type;
-		
+
 		/* ignore empty, dot thingies */
 		if (!entry->d_name || entry->d_name[0] == '.')
 			continue;
-		
+
 		/* we have to copy the buffer from fullpath_s, because
 		 * it returns a static buffer and we are
 		 * recursive*/
 		fp = mu_str_fullpath_s (dirname, entry->d_name);
 		fullpath = g_newa (char, strlen(fp) + 1);
 		strcpy (fullpath, fp);
-		
+
 		d_type = GET_DTYPE (entry, fullpath);
-		
+
 		/* ignore non-links / non-dirs */
 		if (d_type != DT_LNK && d_type != DT_DIR)
-			continue; 
-				
+			continue;
+
 		if (d_type == DT_LNK) {
 			if (unlink (fullpath) != 0) {
 				/* don't use err */
@@ -590,7 +589,7 @@ clear_links (const gchar* dirname, DIR *dir, GError **err)
 	if (errno != 0)
 		g_set_error (err, 0, MU_ERROR_FILE,
 			     "file error: %s", strerror(errno));
-	
+
 	return (rv == FALSE && errno == 0);
 }
 
@@ -600,10 +599,10 @@ mu_maildir_clear_links (const gchar* path, GError **err)
 {
 	DIR *dir;
 	gboolean rv;
-	
+
 	g_return_val_if_fail (path, FALSE);
-	
-	dir = opendir (path);		
+
+	dir = opendir (path);
 	if (!dir) {
 		g_set_error (err, 0, MU_ERROR_FILE_CANNOT_OPEN,
 			     "failed to open %s: %s", path,
@@ -617,20 +616,11 @@ mu_maildir_clear_links (const gchar* path, GError **err)
 	return rv;
 }
 
-
-/*
- * is this a 'new' msg or a 'cur' msg?; if new, we return
- * (in info) a ptr to the info part 
- */
-enum _MsgType { MSG_TYPE_CUR, MSG_TYPE_NEW, MSG_TYPE_OTHER };
-typedef enum _MsgType MsgType;
-
-static MsgType
-check_msg_type (const char *path, char **info)
+MuFlags
+mu_maildir_get_flags_from_path (const char *path)
 {
-	char *dir, *file;
-	MsgType mtype;
-		
+	g_return_val_if_fail (path, MU_FLAG_INVALID);
+
 	/* try to find the info part */
 	/* note that we can use either the ':' or '!' as separator;
 	 * the former is the official, but as it does not work on e.g. VFAT
@@ -640,96 +630,42 @@ check_msg_type (const char *path, char **info)
 	 * mentions the '!' as well as a 'popular choice'
 	 */
 
-	*info = NULL;
-	dir = g_path_get_dirname(path);
-	file = g_path_get_basename(path);
-		
-	if (!(*info = strrchr(file, ':')))
-		*info = strrchr(file, '!');	/* Tinymail */
-	if (*info)
-		++(*info);	/* skip the ':' or '!' */
+	/* we check the dir -- */
+	if (strstr (path, G_DIR_SEPARATOR_S "new" G_DIR_SEPARATOR_S)) {
 
-	if (g_str_has_suffix(dir, G_DIR_SEPARATOR_S "cur")) {
-		if (!*info)
-			g_debug("'cur' file, but no info part: %s", path);
-		mtype = MSG_TYPE_CUR;
-	} else if (g_str_has_suffix(dir, G_DIR_SEPARATOR_S "new")) {
-		if (*info)
-			g_debug("'new' file, ignoring info part: %s", path);
-		mtype = MSG_TYPE_NEW;
-	} else
-		mtype = MSG_TYPE_OTHER;	/* file has been added explicitly as
-					   a single message */
-	if (*info)
-		*info = g_strdup(*info);
+		char *dir, *dir2;
+		MuFlags flags;
 
-	g_free(dir);
-	g_free(file);
+		dir  = g_path_get_dirname (path);
+		dir2 = g_path_get_basename (dir);
 
-	return mtype;
-}
+		if (g_strcmp0 (dir2, "new") == 0)
+			flags = MU_FLAG_NEW;
 
+		g_free (dir);
+		g_free (dir2);
 
-MuMsgFlags
-mu_maildir_get_flags_from_path (const char *path)
-{
-	MuMsgFlags flags;
-	MsgType mtype;
-	char *info = NULL, *cursor;
-
-	g_return_val_if_fail (path, MU_MSG_FLAG_NONE);
-	g_return_val_if_fail (!g_str_has_suffix(path, G_DIR_SEPARATOR_S),
-			      MU_MSG_FLAG_NONE);
-
-	mtype = check_msg_type (path, &info);
-	if (mtype == MSG_TYPE_NEW) {	/* we ignore any new-msg flags */
-		/* note NEW implies UNREAD */
-		flags = MU_MSG_FLAG_NEW | MU_MSG_FLAG_UNREAD;
-		goto leave;
+		/* NOTE: new/ message should not have :2,-stuff, as
+		 * per http://cr.yp.to/proto/maildir.html. If they, do
+		 * we ignore it
+		 */
+		if (flags == MU_FLAG_NEW)
+			return flags;
 	}
 
-	flags = MU_MSG_FLAG_NONE;
-	if ((mtype != MSG_TYPE_CUR && mtype != MSG_TYPE_OTHER) ||
-	    !(info && info[0] == '2' && info[1] == ','))
-		goto leave;
-		
-	for (cursor = info + 2; *cursor; ++cursor)
-		flags |= mu_msg_flag_from_file_char (*cursor);
+	/*  get the file flags */
+	{
+		char *info;
 
-	/* the UNREAD pseudo flag => NEW OR NOT SEEN */
-	if (!(flags & MU_MSG_FLAG_SEEN))
-		flags |= MU_MSG_FLAG_UNREAD;		
-leave:
-	g_free(info);
-	return flags;
-}
-
-/* note: returns static string, non-reentrant */
-static const char*
-get_flags_str_s (MuMsgFlags flags)
-{
-	int i;
-	static char flagstr[7]; 
-
-	i = 0;
-		
-	/* now, determine the flags to use */
-	if (flags & MU_MSG_FLAG_DRAFT)
-		flagstr[i++] = 'D';
-	if (flags & MU_MSG_FLAG_FLAGGED)
-		flagstr[i++] = 'F';
-	if (flags & MU_MSG_FLAG_PASSED)
-		flagstr[i++] = 'P';
-	if (flags & MU_MSG_FLAG_REPLIED)
-		flagstr[i++] = 'R';
-	if (flags & MU_MSG_FLAG_SEEN)
-		flagstr[i++] = 'S';
-	if (flags & MU_MSG_FLAG_TRASHED)
-		flagstr[i++] = 'T';
-		
-	flagstr[i] = '\0';
-
-	return flagstr;
+		info = strrchr (path, '2');
+		if (!info || info == path ||
+		    (info[-1] != ':' && info[-1] != '!') ||
+		    (info[1] != ','))
+			return MU_FLAG_NONE;
+		else
+			return mu_flags_from_str (&info[2],
+						  MU_FLAG_TYPE_MAILFILE);
+	}
 }
 
 
@@ -737,119 +673,192 @@ get_flags_str_s (MuMsgFlags flags)
  * take an exising message path, and return a new path, based on
  * whether it should be in 'new' or 'cur'; ie.
  *
- * /home/user/Maildir/foo/bar/cur/abc:2,F  and flags == MU_MSG_FLAG_NEW
+ * /home/user/Maildir/foo/bar/cur/abc:2,F  and flags == MU_FLAG_NEW
  *     => /home/user/Maildir/foo/bar/new
- * and 
- * /home/user/Maildir/foo/bar/new/abc  and flags == MU_MSG_FLAG_REPLIED
+ * and
+ * /home/user/Maildir/foo/bar/new/abc  and flags == MU_FLAG_REPLIED
  *    => /home/user/Maildir/foo/bar/cur
  *
- * so only difference is whether MuMsgFlags matches MU_MSG_FLAG_NEW is set or not
- * 
- */
-static char*
-get_new_dir_name (const char* oldpath, MuMsgFlags flags)
-{
-	char *newpath, *dirpart;
-		
-	/* g_path_get_dirname is not explicit about whether it ends in
-	 * a dir-separator (\ or /), so we need to check both */
-	const char* cur4 = G_DIR_SEPARATOR_S "cur";
-	const char* cur5 = G_DIR_SEPARATOR_S "cur" G_DIR_SEPARATOR_S;
-	const char* new4 = G_DIR_SEPARATOR_S "new";
-	const char* new5 = G_DIR_SEPARATOR_S "new" G_DIR_SEPARATOR_S;
-		
-	g_return_val_if_fail (oldpath, NULL);
-	/* if MU_MSG_FLAG_NEW is set, it must be the only flag */
-	g_return_val_if_fail (flags & MU_MSG_FLAG_NEW ?
-			      flags == MU_MSG_FLAG_NEW : TRUE, NULL);
-		
-	newpath = g_path_get_dirname (oldpath);
-	if (g_str_has_suffix (newpath, cur4) || g_str_has_suffix (newpath, new4))
-		dirpart = &newpath[strlen(newpath) - strlen(cur4)];
-	else if (g_str_has_suffix (newpath, cur5) || g_str_has_suffix (newpath, new5))
-		dirpart = &newpath[strlen(newpath) - strlen(cur5)];
-	else {
-		g_warning ("invalid maildir path: %s", oldpath);
-		g_free (newpath);
-		return NULL;
-	}
-
-	/* now, copy the desired dir part behind this */
-	if (flags & MU_MSG_FLAG_NEW) 
-		memcpy (dirpart, new4, strlen(new4) + 1);
-	else
-		memcpy (dirpart, cur4, strlen(cur4) + 1);
-		
-	return newpath;
-}
-
-/*
- * get a new filename for the message, based on the new flags; if the
- * message has MU_MSG_FLAG_NEW, it will loose its flags
+ * so only difference is whether MuFlags matches MU_FLAG_NEW is set or not
  *
- */ 
-static char*
-get_new_file_name (const char *oldpath, MuMsgFlags flags)
+ */
+static gchar*
+get_new_path (const char *mdir, const char *mfile, MuFlags flags)
 {
-	gchar *newname, *sep;
-		
-	/* if MU_MSG_FLAG_NEW is set, it must be the only flag */
-	g_return_val_if_fail (flags & MU_MSG_FLAG_NEW ?
-			      flags == MU_MSG_FLAG_NEW : TRUE, NULL);
-		
-	/* the normal separator is ':', but on e.g. vfat, '!' is seen
-	 * as well */
-	newname		= g_path_get_basename (oldpath);
-	if (!newname) {
-		g_warning ("invalid path: '%s'", oldpath);
-		return NULL;
-	}
-
-	if (!(sep = g_strrstr (newname, ":")) &&
-	    !(sep = g_strrstr (newname, "!"))) {
-		g_warning ("not a valid msg file name: '%s'", oldpath);
-		g_free (newname);
-		return NULL;
-	}
-
-	if (flags & MU_MSG_FLAG_NEW)
-		sep[0] = '\0'; /* remove all, including ':' or '!' */
+	if (flags & MU_FLAG_NEW)
+		return g_strdup_printf ("%s%cnew%c%s",
+					mdir, G_DIR_SEPARATOR, G_DIR_SEPARATOR,
+					mfile);
 	else {
-		gchar *tmp;
-		sep[1] = '\0'; /* remove flags, but keep ':' or '!' */
-		sep[flags & MU_MSG_FLAG_NEW ? 0 : 1] = '\0';
-		tmp = newname;
-		newname = g_strdup_printf ("%s2,%s", newname, get_flags_str_s (flags));
-		g_free (tmp);
-	}
+		const char *flagstr;
+		flagstr = mu_flags_to_str_s (flags, MU_FLAG_TYPE_MAILFILE);
 
-	return newname;
+		return g_strdup_printf ("%s%ccur%c%s:2,%s",
+					mdir, G_DIR_SEPARATOR, G_DIR_SEPARATOR,
+					mfile, flagstr);
+	}
 }
 
+
 char*
-mu_maildir_get_path_from_flags (const char *oldpath, MuMsgFlags newflags)
+mu_maildir_get_maildir_from_path (const char* path)
 {
-	char *newname, *newdir, *newpath;
-		
-	g_return_val_if_fail (oldpath, NULL);
-	g_return_val_if_fail (newflags != MU_MSG_FLAG_NONE, NULL);
-	/* if MU_MSG_FLAG_NEW is set, it must be the only flag */
-	g_return_val_if_fail (newflags & MU_MSG_FLAG_NEW ?
-			      newflags == MU_MSG_FLAG_NEW : TRUE, NULL);
-	
-	newname = get_new_file_name (oldpath, newflags);
-	if (!newname)
-		return NULL;
+	gchar *mdir;
 
-	newdir = get_new_dir_name (oldpath, newflags);
-	if (!newdir) {
-		g_free (newname);
+	/* determine the maildir */
+	mdir = g_path_get_dirname (path);
+	if (!g_str_has_suffix (mdir, "cur") && !g_str_has_suffix (mdir, "new")) {
+		g_warning ("%s: not a valid maildir path: %s",
+			   __FUNCTION__, path);
+		g_free (mdir);
 		return NULL;
 	}
-				
-	newpath = g_strdup_printf ("%s%c%s", newdir, G_DIR_SEPARATOR, newname);
-	g_free (newname);
-	g_free (newdir);
-		
+
+	/* remove the 'cur' or 'new' */
+	mdir[strlen(mdir) - 4] = '\0';
+
+	return mdir;
+}
+
+
+
+char*
+mu_maildir_get_new_path (const char *oldpath, const char *new_mdir,
+			 MuFlags newflags)
+{
+	char *mfile, *mdir, *newpath, *cur;
+
+	g_return_val_if_fail (oldpath, NULL);
+
+	mfile = newpath = NULL;
+
+	/* determine the maildir */
+	mdir = mu_maildir_get_maildir_from_path (oldpath);
+	if (!mdir)
+		return NULL;
+
+	/* determine the name of the mailfile, stripped of its flags */
+	mfile = g_path_get_basename (oldpath);
+	for (cur = &mfile[strlen(mfile)-1]; cur > mfile; --cur) {
+		if ((*cur == ':' || *cur == '!') &&
+		    (cur[1] == '2' && cur[2] == ',')) {
+			cur[0] = '\0'; /* strip the flags */
+			break;
+		}
+	}
+
+	newpath = get_new_path (new_mdir ? new_mdir : mdir,
+				mfile, newflags);
+
+	g_free (mfile);
+	g_free (mdir);
+
 	return newpath;
 }
+
+static gboolean
+msg_move_check_pre (const gchar *src, const gchar *dst, GError **err)
+{
+	if (!g_path_is_absolute(src)) {
+		g_set_error (err, 0, MU_ERROR_FILE,
+			     "source is not an absolute path: '%s'", src);
+		return FALSE;
+	}
+
+	if (!g_path_is_absolute(dst)) {
+		g_set_error (err, 0, MU_ERROR_FILE,
+			     "target is not an absolute path: '%s'", dst);
+		return FALSE;
+	}
+
+	if (access (src, R_OK) != 0) {
+		g_set_error (err, 0, MU_ERROR_FILE, "cannot read %s",
+			     src);
+		return FALSE;
+	}
+
+	if (access (dst, F_OK) == 0) {
+		g_set_error (err, 0, MU_ERROR_FILE, "%s already exists",
+			     dst);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+msg_move_check_post (const char *src, const char *dst, GError **err)
+{
+	/* double check -- is the target really there? */
+	if (access (dst, F_OK) != 0) {
+		g_set_error (err, 0, MU_ERROR_FILE, "can't find target (%s)",
+			     dst);
+		return FALSE;
+	}
+
+	if (access (src, F_OK) == 0) {
+		g_set_error (err, 0, MU_ERROR_FILE, "source is still there (%s)",
+			     src);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+static gboolean
+msg_move (const char* src, const char *dst, GError **err)
+{
+	if (!msg_move_check_pre (src, dst, err))
+		return FALSE;
+
+	if (rename (src, dst) != 0) {
+		g_set_error (err, 0, MU_ERROR_FILE, "error moving %s to %s",
+			     src, dst);
+		return FALSE;
+	}
+
+	if (!msg_move_check_post (src, dst, err))
+		return FALSE;
+
+	return TRUE;
+}
+
+gchar*
+mu_maildir_move_message (const char* oldpath, const char* targetmdir,
+			 MuFlags newflags, gboolean ignore_dups,
+			 GError **err)
+{
+	char *newfullpath;
+	gboolean rv;
+	gboolean src_is_target;
+
+	g_return_val_if_fail (oldpath, FALSE);
+
+	newfullpath = mu_maildir_get_new_path (oldpath, targetmdir,
+					       newflags);
+	if (!newfullpath) {
+		g_set_error (err, 0, MU_ERROR_FILE,
+			     "failed to determine target full path");
+		return FALSE;
+	}
+
+
+	src_is_target = (g_strcmp0 (oldpath, newfullpath) == 0);
+
+	if (!ignore_dups && src_is_target) {
+		g_set_error (err, 0, MU_ERROR_FILE_TARGET_EQUALS_SOURCE,
+			     "target equals source");
+		return FALSE;
+	}
+
+	if (!src_is_target) {
+		rv = msg_move (oldpath, newfullpath, err);
+		if (!rv) {
+			g_free (newfullpath);
+			return NULL;
+		}
+	}
+
+	return newfullpath;
+}
