@@ -1,5 +1,5 @@
-/* 
-** Copyright (C) 2008-2010 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+/*
+** Copyright (C) 2008-2012 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -13,8 +13,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software Foundation,
-** Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
-**  
+** Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+**
 */
 
 #ifndef __MU_MAILDIR_H__
@@ -23,26 +23,29 @@
 #include <glib.h>
 #include <time.h>
 #include <sys/types.h>          /* for mode_t */
-#include <mu-util.h> 
-#include <mu-msg-flags.h>
+#include <mu-util.h>
+#include <mu-flags.h>
 
 
 G_BEGIN_DECLS
 
 /**
- * create a new maildir. Note, if the function fails 'halfway', it
- * will *not* try to remove the parts the were created. it *will*
- * create any parent dirs that are not yet existant.
- * 
- * 
+ * create a new maildir. if parts of the maildir already exists, those
+ * will simply be ignored. IOW, if you try to create the same maildir
+ * twice, the second will simply be a no-op (without any errors).
+ * Note, if the function fails 'halfway', it will *not* try to remove
+ * the parts the were created. it *will* create any parent dirs that
+ * are not yet existent.
+ *
+ *
  * @param path the path (missing components will be created, as in 'mkdir -p')
  * @param mode the file mode (e.g., 0755)
  * @param noindex add a .noindex file to the maildir, so it will be excluded
  * from indexing by 'mu index'
- * @param err if function returns FALSE, err may contain extra
- * information. if err is NULL, does nothing
- * 
- * @return TRUE if creation succeeded, FALSE otherwise
+ * @param err if function returns FALSE, receives error
+ * information. err may be NULL.
+ *
+ * @return TRUE if creation succeeded (or already existed), FALSE otherwise
  */
 gboolean mu_maildir_mkdir (const char* path, mode_t mode, gboolean noindex,
 			   GError **err);
@@ -50,17 +53,18 @@ gboolean mu_maildir_mkdir (const char* path, mode_t mode, gboolean noindex,
 
 /**
  * create a symbolic link to a mail message
- * 
+ *
  * @param src the full path to the source message
  * @param targetpath the path to the target maildir; ie., *not*
  * MyMaildir/cur, but just MyMaildir/. The function will figure out
- * the correct subdir then. 
+ * the correct subdir then.
  * @param err if function returns FALSE, err may contain extra
  * information. if err is NULL, does nothing
- * 
- * @return 
+ *
+ * @return
  */
-gboolean mu_maildir_link   (const char* src, const char *targetpath, GError **err);
+gboolean mu_maildir_link   (const char* src, const char *targetpath,
+			    GError **err);
 
 /**
  * MuMaildirWalkMsgCallback -- callback function for
@@ -71,15 +75,16 @@ gboolean mu_maildir_link   (const char* src, const char *targetpath, GError **er
  * the maildir "foo/bar". Then, the information from 'stat' of this
  * file (see stat(3)), and a user_data pointer
  */
-typedef MuResult (*MuMaildirWalkMsgCallback)
-(const char* fullpath, const char* mdir, struct stat *statinfo, void *user_data);
+typedef MuError (*MuMaildirWalkMsgCallback)
+  (const char* fullpath, const char* mdir, struct stat *statinfo,
+   void *user_data);
 
 /**
  * MuPathWalkDirCallback -- callback function for mu_path_walk_maildir; see the
  * documentation there. It will be called each time a dir is entered or left,
- * with 'enter' being TRUE upon entering, FALSE otherwise 
+ * with 'enter' being TRUE upon entering, FALSE otherwise
  */
-typedef MuResult (*MuMaildirWalkDirCallback)
+typedef MuError (*MuMaildirWalkDirCallback)
      (const char* fullpath, gboolean enter, void *user_data);
 
 /**
@@ -96,25 +101,25 @@ typedef MuResult (*MuMaildirWalkDirCallback)
  * mu_walk_maildir stops if the callbacks return something different
  * from MU_OK. For example, it can return MU_STOP to stop the scan, or
  * some error.
- * 
+ *
  * @param path the maildir path to scan
  * @param cb_msg the callback function called for each msg
  * @param cb_dir the callback function called for each dir
  * @param data user data pointer
- * 
- * @return a scanner result; MU_OK if everything went ok, 
+ *
+ * @return a scanner result; MU_OK if everything went ok,
  * MU_STOP if we want to stop, or MU_ERROR in
  * case of error
  */
-MuResult mu_maildir_walk (const char *path, MuMaildirWalkMsgCallback cb_msg, 
-			  MuMaildirWalkDirCallback cb_dir, void *data);
+MuError mu_maildir_walk (const char *path, MuMaildirWalkMsgCallback cb_msg,
+			 MuMaildirWalkDirCallback cb_dir, void *data);
 /**
  * recursively delete all the symbolic links in a directory tree
- * 
+ *
  * @param dir top dir
  * @param err if function returns FALSE, err may contain extra
  * information. if err is NULL, does nothing
- * 
+ *
  * @return TRUE if it worked, FALSE in case of error
  */
 gboolean mu_maildir_clear_links (const gchar* dir, GError **err);
@@ -129,29 +134,70 @@ gboolean mu_maildir_clear_links (const gchar* dir, GError **err);
  *
  * @param pathname of a mailfile; it does not have to refer to an
  * actual message
- * 
+ *
  * @return the flags, or MU_MSG_FILE_FLAG_UNKNOWN in case of error
  */
-MuMsgFlags mu_maildir_get_flags_from_path (const char* pathname);
+MuFlags mu_maildir_get_flags_from_path (const char* pathname);
 
 /**
  * get the new pathname for a message, based on the old path and the
- * new flags. Note that setting/removing the MU_MSG_FLAG_NEW will
- * change the directory in which a message lives. The flags are as
- * specified in http://cr.yp.to/proto/maildir.html, plus
- * MU_MSG_FLAG_NEW for new messages, ie the ones that live in
- * new/. The flags are logically OR'ed. Note that the file does not
- * have to exist; the flags are based on the path only.
- * 
+ * new flags and (optionally) a new maildir. Note that
+ * setting/removing the MU_FLAG_NEW will change the directory in which
+ * a message lives. The flags are as specified in
+ * http://cr.yp.to/proto/maildir.html, plus MU_FLAG_NEW for new
+ * messages, ie the ones that live in new/. The flags are logically
+ * OR'ed. Note that the file does not have to exist; the flags are
+ * based on the path only.
+ *
+ *
  * @param oldpath the old (current) full path to the message
- * (including the filename) *
- * @param newflags the new flags for this message
- * 
+ * (including the filename)
+ * @param new_mdir the new maildir for this message, or NULL to keep
+ * it in the current one. The maildir is the absolute file system
+ * path, without the 'cur' or 'new'
+ * @param new_flags the new flags for this message
+ *
  * @return a new path name; use g_free when done with. NULL in case of
  * error.
  */
-char* mu_maildir_get_path_from_flags (const char *oldpath,
-				       MuMsgFlags newflags);
+char* mu_maildir_get_new_path (const char *oldpath, const char *new_mdir,
+			       MuFlags new_flags);
+
+/**
+ * get the maildir for a certain message path, ie, the path *before*
+ * cur/ or new/
+ *
+ * @param path path for some message
+ *
+ * @return the maildir (free with g_free), or NULL in case of error
+ */
+char* mu_maildir_get_maildir_from_path (const char* path);
+
+
+/**
+ * move a message file to another maildir; the function returns the full
+ * path to the new message.
+ *
+ * @param msgpath an absolute file system path to an existing message in an
+ * actual maildir
+ * @param targetmdir the target maildir; note that this the base-level
+ * Maildir, ie. /home/user/Maildir/archive, and must _not_ include the
+ * 'cur' or 'new' part. Note that the target maildir must be on the
+ * same filesystem. If you specify NULL for targetmdir, only the flags
+ * of the message are affected; note that this may still involve a
+ * moved to another directory (say, from new/ to cur/)
+ * @param flags to set for the target (influences the filename, path)
+ * @param ignore_dups whether to silent ignore the src=target case (and return TRUE)
+ * @param err (may be NULL) may contain error information; note if the
+ * function return FALSE, err is not set for all error condition
+ * (ie. not for parameter errors)
+ *
+ * @return return the full path name of the target file (g_free) if
+ * the move succeeded, NULL otherwise
+ */
+gchar* mu_maildir_move_message (const char* oldpath, const char* targetmdir,
+				MuFlags newflags, gboolean ignore_dups,
+				GError **err);
 
 G_END_DECLS
 
