@@ -37,7 +37,6 @@
 #endif 	/*PATH_MAX */
 
 #include <gmime/gmime.h>
-
 #include "mu-util.h"
 #include "mu-str.h"
 #include "mu-maildir.h"
@@ -49,27 +48,25 @@ static gboolean init_file_metadata (MuMsgFile *self, const char* path,
 static gboolean init_mime_msg (MuMsgFile *msg, const char *path, GError **err);
 
 
-
-
-MuMsgFile*   
+MuMsgFile*
 mu_msg_file_new (const char* filepath, const char *mdir, GError **err)
 {
 	MuMsgFile *self;
 
 	g_return_val_if_fail (filepath, NULL);
-			
-	self = g_slice_new0 (MuMsgFile);	
-	
+
+	self = g_slice_new0 (MuMsgFile);
+
 	if (!init_file_metadata (self, filepath, mdir, err)) {
 		mu_msg_file_destroy (self);
 		return NULL;
 	}
-	
+
 	if (!init_mime_msg (self, filepath, err)) {
 		mu_msg_file_destroy (self);
 		return NULL;
 	}
-	
+
 	return self;
 }
 
@@ -82,10 +79,19 @@ mu_msg_file_destroy (MuMsgFile *self)
 
 	if (self->_mime_msg)
 		g_object_unref (self->_mime_msg);
-	
+
+	mu_str_free_list (self->_free_later);
+
 	g_slice_free (MuMsgFile, self);
 }
 
+static const gchar*
+free_string_later (MuMsgFile *self, gchar *str)
+{
+	self->_free_later = g_slist_prepend (self->_free_later, str);
+	return str;
+}
+
 
 static gboolean
 init_file_metadata (MuMsgFile *self, const char* path, const gchar* mdir,
@@ -106,19 +112,19 @@ init_file_metadata (MuMsgFile *self, const char* path, const gchar* mdir,
 			     path, strerror(errno));
 		return FALSE;
 	}
-	
+
 	if (!S_ISREG(statbuf.st_mode)) {
 		g_set_error (err, 0, MU_ERROR_FILE,
 			     "not a regular file: %s", path);
 		return FALSE;
 	}
-	
+
 	self->_timestamp = statbuf.st_mtime;
 	self->_size	 = (size_t)statbuf.st_size;
 
 	strncpy (self->_path, path, PATH_MAX);
-	strncpy (self->_maildir, mdir ? mdir : "", PATH_MAX); 
-	
+	strncpy (self->_maildir, mdir ? mdir : "", PATH_MAX);
+
 	return TRUE;
 }
 
@@ -129,7 +135,7 @@ get_mime_stream (MuMsgFile *self, const char *path, GError **err)
 {
 	FILE *file;
 	GMimeStream *stream;
-	
+
 	file = fopen (path, "r");
 	if (!file) {
 		g_set_error (err, 0, MU_ERROR_FILE,
@@ -137,7 +143,7 @@ get_mime_stream (MuMsgFile *self, const char *path, GError **err)
 			     path, strerror (errno));
 		return NULL;
 	}
-	
+
 	stream = g_mime_stream_file_new (file);
 	if (!stream) {
 		g_set_error (err, 0, MU_ERROR_GMIME,
@@ -155,11 +161,11 @@ init_mime_msg (MuMsgFile *self, const char* path, GError **err)
 {
 	GMimeStream *stream;
 	GMimeParser *parser;
-	
+
 	stream = get_mime_stream (self, path, err);
 	if (!stream)
 		return FALSE;
-	
+
 	parser = g_mime_parser_new_with_stream (stream);
 	g_object_unref (stream);
 	if (!parser) {
@@ -168,7 +174,7 @@ init_mime_msg (MuMsgFile *self, const char* path, GError **err)
 			     __FUNCTION__, path);
 		return FALSE;
 	}
-	
+
 	self->_mime_msg = g_mime_parser_construct_message (parser);
 	g_object_unref (parser);
 	if (!self->_mime_msg) {
@@ -187,7 +193,7 @@ get_recipient (MuMsgFile *self, GMimeRecipientType rtype)
 {
 	char *recip;
 	InternetAddressList *recips;
-	
+
 	recips = g_mime_message_get_recipients (self->_mime_msg, rtype);
 
 	/* FALSE --> don't encode */
@@ -197,7 +203,7 @@ get_recipient (MuMsgFile *self, GMimeRecipientType rtype)
 		g_debug ("invalid recipient in %s\n", self->_path);
 		mu_str_asciify_in_place (recip); /* ugly... */
 	}
-		
+
 	if (mu_str_is_empty(recip)) {
 		g_free (recip);
 		return NULL;
@@ -207,68 +213,79 @@ get_recipient (MuMsgFile *self, GMimeRecipientType rtype)
 }
 
 
-
 static gboolean
-part_looks_like_attachment (GMimeObject *part)
+looks_like_attachment (GMimeObject *part)
 {
-	GMimeContentDisposition *disp;
 	const char *str;
-	
-	disp  = g_mime_object_get_content_disposition (part);
-	if (!GMIME_IS_CONTENT_DISPOSITION(disp))
-		return FALSE; /* no content disp? prob not
-			       * an attachment. */
-	
-	str = g_mime_content_disposition_get_disposition (disp);
+	GMimeContentDisposition *disp;
+	GMimeContentType *ct;
 
-	/* ok, it says it's an attachment, so it probably is... */
+	disp = g_mime_object_get_content_disposition (GMIME_OBJECT(part));
+	if (!GMIME_IS_CONTENT_DISPOSITION(disp))
+		return FALSE;
+
+	str = g_mime_content_disposition_get_disposition (disp);
 	if (!str)
+		return FALSE;
+
+	ct = g_mime_object_get_content_type (part);
+	if (!ct)
+		return FALSE; /* ignore this part... */
+
+	/* note, some mailers use ATTACHMENT, INLINE instead of their
+	 * more common lower-case counterparts */
+	if (g_ascii_strcasecmp(str, GMIME_DISPOSITION_ATTACHMENT) == 0)
 		return TRUE;
-	if (strcmp (str, GMIME_DISPOSITION_ATTACHMENT) == 0)
-		return TRUE;
-	else if (strcmp (str, GMIME_DISPOSITION_INLINE) == 0) {
-		/* inline-images are also considered attachments... */
-		GMimeContentType *ct;
-		ct = g_mime_object_get_content_type (part);
-		if (ct)
-			return g_mime_content_type_is_type
-				(ct, "image", "*");
+
+	if (g_ascii_strcasecmp(str, GMIME_DISPOSITION_INLINE) == 0) {
+		/* some inline parts are also considered attachments... */
+		int i;
+		const char* att_types[][2] = {
+			{"image", "*"},
+			{"application", "*"},
+			{"message", "*"}};
+
+		for (i = 0; i != G_N_ELEMENTS (att_types); ++i)
+			if (g_mime_content_type_is_type (ct,
+							 att_types[i][0],
+							 att_types[i][1]))
+				return TRUE; /* looks like an attachment */
 	}
-	
-	return FALSE;
+
+	return FALSE; /* does not look like an attachment */
 }
-					  
+
 
 static void
-msg_cflags_cb (GMimeObject *parent, GMimeObject *part, MuMsgFlags *flags)
+msg_cflags_cb (GMimeObject *parent, GMimeObject *part, MuFlags *flags)
 {
-	if (*flags & MU_MSG_FLAG_HAS_ATTACH)
+	if (*flags & MU_FLAG_HAS_ATTACH)
 		return;
-	
+
 	if (!GMIME_IS_PART(part))
 		return;
-	
-	if (part_looks_like_attachment(part))
-		*flags |= MU_MSG_FLAG_HAS_ATTACH;
+
+	if (!(*flags & MU_FLAG_HAS_ATTACH) && looks_like_attachment(part))
+		*flags |= MU_FLAG_HAS_ATTACH;
 }
 
 
 
-static MuMsgFlags
+static MuFlags
 get_content_flags (MuMsgFile *self)
 {
 	GMimeContentType *ctype;
-	MuMsgFlags flags;
+	MuFlags flags;
 	GMimeObject *part;
 
 	if (!GMIME_IS_MESSAGE(self->_mime_msg))
-		return MU_MSG_FLAG_NONE;
+		return MU_FLAG_NONE;
 
 	flags = 0;
 	g_mime_message_foreach (self->_mime_msg,
-				(GMimeObjectForeachFunc)msg_cflags_cb, 
+				(GMimeObjectForeachFunc)msg_cflags_cb,
 				&flags);
-	
+
 	/* note: signed or encrypted status for a message is determined by
 	 *  the top-level mime-part
 	 */
@@ -278,15 +295,15 @@ get_content_flags (MuMsgFile *self)
 		if (!ctype) {
 			g_warning ("not a content type!");
 			return 0;
-		}	
-		
+		}
+
 		if (ctype) {
 			if (g_mime_content_type_is_type
-			    (ctype,"*", "signed")) 
-				flags |= MU_MSG_FLAG_SIGNED;
+			    (ctype,"*", "signed"))
+				flags |= MU_FLAG_SIGNED;
 			if (g_mime_content_type_is_type
-			    (ctype,"*", "encrypted")) 
-				flags |= MU_MSG_FLAG_ENCRYPTED;
+			    (ctype,"*", "encrypted"))
+				flags |= MU_FLAG_ENCRYPTED;
 		}
 	} else
 		g_warning ("no top level mime part found");
@@ -295,16 +312,21 @@ get_content_flags (MuMsgFile *self)
 }
 
 
-static MuMsgFlags
+static MuFlags
 get_flags (MuMsgFile *self)
 {
-	MuMsgFlags flags;
-	
-	g_return_val_if_fail (self, MU_MSG_FLAG_NONE);
+	MuFlags flags;
+
+	g_return_val_if_fail (self, MU_FLAG_INVALID);
 
 	flags = mu_maildir_get_flags_from_path (self->_path);
 	flags |= get_content_flags (self);
-	
+
+	/* pseudo-flag --> unread means either NEW or NOT SEEN, just
+	 * for searching convenience */
+	if ((flags & MU_FLAG_NEW) || !(flags & MU_FLAG_SEEN))
+		flags |= MU_FLAG_UNREAD;
+
 	return flags;
 }
 
@@ -338,14 +360,15 @@ get_prio_header_field (MuMsgFile *self)
 
 	obj = GMIME_OBJECT(self->_mime_msg);
 
-	str = g_mime_object_get_header (obj, "X-Priority");
+	str = g_mime_object_get_header (obj, "Precedence");
 	if (!str)
-		str = g_mime_object_get_header (obj, "X-MSMail-Priority");
+		str = g_mime_object_get_header (obj, "X-Priority");
 	if (!str)
 		str = g_mime_object_get_header (obj, "Importance");
-	if (!str)
-		str = g_mime_object_get_header (obj, "Precedence");
-	if (str) 
+	/* NOTE: "X-MSMail-Priority" is never seen without "X-Priority" */
+	/* if (!str) */
+	/* 	str = g_mime_object_get_header (obj, "X-MSMail-Priority"); */
+	if (str)
 		return (to_lower(g_strdup(str)));
 	else
 		return NULL;
@@ -363,7 +386,7 @@ parse_prio_str (const char* priostr)
 		{ "high",	MU_MSG_PRIO_HIGH },
 		{ "1",		MU_MSG_PRIO_HIGH },
 		{ "2",		MU_MSG_PRIO_HIGH },
-		
+
 		{ "normal",	MU_MSG_PRIO_NORMAL },
 		{ "3",		MU_MSG_PRIO_NORMAL },
 
@@ -377,7 +400,7 @@ parse_prio_str (const char* priostr)
 	for (i = 0; i != G_N_ELEMENTS(str_prio); ++i)
 		if (g_strstr_len (priostr, -1, str_prio[i]._str) != NULL)
 			return str_prio[i]._prio;
-	
+
 	/* e.g., last-fm uses 'fm-user'... as precedence */
 	return MU_MSG_PRIO_NORMAL;
 }
@@ -393,7 +416,7 @@ get_prio (MuMsgFile *self)
 	priostr = get_prio_header_field (self);
 	if (!priostr)
 		return MU_MSG_PRIO_NORMAL;
-	
+
 	prio = parse_prio_str (priostr);
 	g_free (priostr);
 
@@ -408,45 +431,26 @@ struct _GetBodyData {
 typedef struct _GetBodyData GetBodyData;
 
 
-static gboolean
-looks_like_attachment (GMimeObject *part)
-{
-	const char *str;
-	GMimeContentDisposition *disp;
-	
-	disp = g_mime_object_get_content_disposition (GMIME_OBJECT(part));
-	if (!GMIME_IS_CONTENT_DISPOSITION(disp))
-		return FALSE;  
-
-	str = g_mime_content_disposition_get_disposition (disp);
-	if (!str)
-		return FALSE;
-	
-	if (strcmp(str,GMIME_DISPOSITION_INLINE) == 0)
-		return FALSE; /* inline, so it's not an attachment */
-	
-	return TRUE; /* looks like an attachment */
-}
 
 static void
 get_body_cb (GMimeObject *parent, GMimeObject *part, GetBodyData *data)
 {
-	GMimeContentType *ct;		
+	GMimeContentType *ct;
 
 	/* already found what we're looking for? */
 	if ((data->_want_html && data->_html_part != NULL) ||
 	    (!data->_want_html && data->_txt_part != NULL))
 		return;
-	
+
 	ct = g_mime_object_get_content_type (part);
 	if (!GMIME_IS_CONTENT_TYPE(ct)) {
 		g_warning ("not a content type!");
 		return;
 	}
-	
+
 	if (looks_like_attachment (part))
 		return; /* not the body */
-	
+
 	/* is it right content type? */
 	if (g_mime_content_type_is_type (ct, "text", "plain"))
 		data->_txt_part = part;
@@ -454,7 +458,7 @@ get_body_cb (GMimeObject *parent, GMimeObject *part, GetBodyData *data)
 		data->_html_part = part;
 	else
 		return; /* wrong type */
-}	
+}
 
 
 
@@ -465,20 +469,20 @@ convert_to_utf8 (GMimePart *part, char *buffer)
 	GMimeContentType *ctype;
 	const char* charset;
 	unsigned char *cur;
-	
+
 	/* optimization: if the buffer is plain ascii, no conversion
 	 * is done... */
 	for (cur = (unsigned char*)buffer; *cur && *cur < 0x80; ++cur);
 	if (*cur == '\0')
 		return buffer;
-	
+
 	ctype = g_mime_object_get_content_type (GMIME_OBJECT(part));
 	g_return_val_if_fail (GMIME_IS_CONTENT_TYPE(ctype), NULL);
-	
+
 	charset = g_mime_content_type_get_parameter (ctype, "charset");
-	if (charset) 
+	if (charset)
 		charset = g_mime_charset_iconv_name (charset);
-	
+
 	/* of course, the charset specified may be incorrect... */
 	if (charset) {
 		char *utf8 = mu_str_convert_to_utf8 (buffer, charset);
@@ -500,10 +504,10 @@ stream_to_string (GMimeStream *stream, size_t buflen)
 {
 	char *buffer;
 	ssize_t bytes;
-	
+
 	buffer = g_new(char, buflen + 1);
 	g_mime_stream_reset (stream);
-	
+
 	/* we read everything in one go */
 	bytes = g_mime_stream_read (stream, buffer, buflen);
 	if (bytes < 0) {
@@ -511,15 +515,15 @@ stream_to_string (GMimeStream *stream, size_t buflen)
 		g_free (buffer);
 		return NULL;
 	}
-	
-	buffer[bytes]='\0'; 
+
+	buffer[bytes]='\0';
 
 	return buffer;
 }
 
 
-static gchar*
-part_to_string (GMimePart *part, gboolean *err)
+gchar*
+mu_msg_mime_part_to_string (GMimePart *part, gboolean *err)
 {
 	GMimeDataWrapper *wrapper;
 	GMimeStream *stream = NULL;
@@ -528,7 +532,7 @@ part_to_string (GMimePart *part, gboolean *err)
 
 	*err = TRUE;
 	g_return_val_if_fail (GMIME_IS_PART(part), NULL);
-	
+
 	wrapper = g_mime_part_get_content_object (part);
 	if (!wrapper) {
 		/* this happens with invalid mails */
@@ -547,58 +551,131 @@ part_to_string (GMimePart *part, gboolean *err)
 		*err = FALSE;
 		goto cleanup;
 	}
-	
+
 	buffer = stream_to_string (stream, (size_t)buflen);
-	
+
 	/* convert_to_utf8 will free the old 'buffer' if needed */
 	buffer = convert_to_utf8 (part, buffer);
-	
+
 	*err = FALSE;
-	
-cleanup:				
+
+cleanup:
 	if (stream)
 		g_object_unref (G_OBJECT(stream));
-	
+
 	return buffer;
 }
 
 
+GMimePart*
+mu_msg_mime_get_body_part (GMimeMessage *msg, gboolean want_html)
+{
+	GetBodyData data;
+
+	g_return_val_if_fail (GMIME_IS_MESSAGE(msg), NULL);
+
+	memset (&data, 0, sizeof(GetBodyData));
+	data._want_html = want_html;
+
+	g_mime_message_foreach (msg,
+				(GMimeObjectForeachFunc)get_body_cb,
+				&data);
+	if (want_html)
+		return (GMimePart*)data._html_part;
+	else
+		return (GMimePart*)data._txt_part;
+}
+
+
+
 static char*
 get_body (MuMsgFile *self, gboolean want_html)
 {
-	GetBodyData data;
-	char *str;
-	gboolean err;
-	
+	GMimePart *part;
+
 	g_return_val_if_fail (self, NULL);
 	g_return_val_if_fail (GMIME_IS_MESSAGE(self->_mime_msg), NULL);
-	
-	memset (&data, 0, sizeof(GetBodyData));
-	data._want_html = want_html;
 
-	err = FALSE;
-	g_mime_message_foreach (self->_mime_msg,
-				(GMimeObjectForeachFunc)get_body_cb,
-				&data);
-	if (want_html)
-		str = data._html_part ?
-			part_to_string (GMIME_PART(data._html_part), &err) :
-			NULL; 
-	else
-		str = data._txt_part ?
-			part_to_string (GMIME_PART(data._txt_part), &err) :
-			NULL;
+	part = mu_msg_mime_get_body_part (self->_mime_msg, want_html);
+	if (GMIME_IS_PART(part)) {
+		gboolean err;
+		gchar *str;
 
-	/* note, str may be NULL (no body), but that's not necessarily
-	 * an error; we only warn when an actual error occured */
-	if (err) 
-		g_warning ("error occured while retrieving %s body" 
-			   "for message %s",
-			   want_html ? "html" : "text", self->_path);
-	return str;
+		err = FALSE;
+		str = mu_msg_mime_part_to_string (part, &err);
+
+		/* note, str may be NULL (no body), but that's not necessarily
+		 * an error; we only warn when an actual error occured */
+		if (err)
+			g_warning ("error occured while retrieving %s body "
+				   "for message %s",
+				   want_html ? "html" : "text", self->_path);
+		return str;
+	}
+
+	return NULL;
 }
 
 
+static void
+append_text (GMimeObject *parent, GMimeObject *part, gchar **txt)
+{
+	GMimeContentType *ct;
+	gchar *parttxt, *tmp;
+	gboolean err;
+
+	if (!GMIME_IS_PART(part))
+		return;
+
+	ct = g_mime_object_get_content_type (part);
+	if (!GMIME_IS_CONTENT_TYPE(ct)) {
+		g_warning ("%s: no content-type for part", __FUNCTION__);
+		return;
+	}
+
+	/* is it right content type? */
+	if (!g_mime_content_type_is_type (ct, "text", "plain"))
+		return; /* nope */
+
+	parttxt = mu_msg_mime_part_to_string (GMIME_PART(part), &err);
+	if (err) {
+		g_warning ("%s: could not get text for part", __FUNCTION__);
+		return;
+	}
+
+	/* it's a text part -- append it! note, we ignore the
+	 * disposition here. */
+	tmp = *txt;
+	if (*txt) {
+		*txt = g_strconcat (*txt, parttxt, NULL);
+		g_free (parttxt);
+	} else
+		*txt = parttxt;
+
+	g_free (tmp);
+}
+
+/* instead of just the body, this function returns a concatenation of
+ * all text/plain parts with inline disposition
+ */
+static char*
+get_concatenated_text (MuMsgFile *self)
+{
+	char *txt;
+
+	g_return_val_if_fail (self, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE(self->_mime_msg), NULL);
+
+	txt = NULL;
+	g_mime_message_foreach (self->_mime_msg,
+				(GMimeObjectForeachFunc)append_text,
+				&txt);
+
+	return txt;
+}
+
+
+
 
 static gboolean
 contains (GSList *lst, const char *str)
@@ -617,29 +694,29 @@ get_references  (MuMsgFile *self)
 	const char *str;
 	unsigned u;
 	const char *headers[] = { "References", "In-reply-to", NULL };
-	
+
 	for (msgids = NULL, u = 0; headers[u]; ++u) {
 
 		const GMimeReferences *cur;
 		GMimeReferences *mime_refs;
-		
-		str = g_mime_object_get_header (GMIME_OBJECT(self->_mime_msg),
-						headers[u]);
+
+		str = mu_msg_file_get_header (self, headers[u]);
 		if (!str)
 			continue;
-		
+
 		mime_refs = g_mime_references_decode (str);
 		for (cur = mime_refs; cur; cur = g_mime_references_get_next(cur)) {
 			const char* msgid;
 			msgid = g_mime_references_get_message_id (cur);
 			/* don't include duplicates */
 			if (msgid && !contains (msgids, msgid))
-				msgids = g_slist_prepend (msgids, g_strdup (msgid));
+				/* explicitly ensure it's utf8-safe, as GMime
+				 * does not ensure that */
+				msgids = g_slist_prepend (msgids, g_strdup((msgid)));
 		}
-
 		g_mime_references_free (mime_refs);
 	}
-	
+
 	return g_slist_reverse (msgids);
 }
 
@@ -647,12 +724,13 @@ get_references  (MuMsgFile *self)
 static GSList*
 get_tags (MuMsgFile *self)
 {
-	GMimeObject *obj;
-	
-	obj = GMIME_OBJECT(self->_mime_msg);
+	const char *hdr;
 
-	return mu_str_to_list (g_mime_object_get_header
-			       (obj, "X-Label"), ',', TRUE);	
+	hdr = mu_msg_file_get_header (self, "X-Label");
+	if (!hdr)
+		return NULL;
+
+	return mu_str_to_list (hdr, ',', TRUE);
 }
 
 
@@ -666,7 +744,7 @@ maybe_cleanup (const char* str, const char *path, gboolean *do_free)
 		return (char*)str;
 
 	g_debug ("invalid utf8 in %s", path);
-	
+
 	if (*do_free)
 		return mu_str_asciify_in_place ((char*)str);
 	else {
@@ -678,6 +756,18 @@ maybe_cleanup (const char* str, const char *path, gboolean *do_free)
 }
 
 
+G_GNUC_CONST static GMimeRecipientType
+recipient_type (MuMsgFieldId mfid)
+{
+	switch (mfid) {
+	case MU_MSG_FIELD_ID_BCC: return GMIME_RECIPIENT_TYPE_BCC;
+	case MU_MSG_FIELD_ID_CC : return GMIME_RECIPIENT_TYPE_CC;
+	case MU_MSG_FIELD_ID_TO : return GMIME_RECIPIENT_TYPE_TO;
+	default: g_return_val_if_reached (-1);
+	}
+}
+
+
 char*
 mu_msg_file_get_str_field (MuMsgFile *self, MuMsgFieldId mfid,
 			   gboolean *do_free)
@@ -686,41 +776,39 @@ mu_msg_file_get_str_field (MuMsgFile *self, MuMsgFieldId mfid,
 	g_return_val_if_fail (mu_msg_field_is_string(mfid), NULL);
 
 	*do_free = FALSE; /* default */
-	
+
 	switch (mfid) {
 
-	case MU_MSG_FIELD_ID_BCC: *do_free = TRUE;
-		return get_recipient (self, GMIME_RECIPIENT_TYPE_BCC);
+	case MU_MSG_FIELD_ID_EMBEDDED_TEXT: *do_free = TRUE;
+		return NULL; /* FIXME */
+
+	case MU_MSG_FIELD_ID_BCC:
+	case MU_MSG_FIELD_ID_CC:
+	case MU_MSG_FIELD_ID_TO: *do_free = TRUE;
+		return get_recipient (self, recipient_type(mfid));
 
 	case MU_MSG_FIELD_ID_BODY_TEXT: *do_free = TRUE;
-		return get_body (self, FALSE);
-
+		return get_concatenated_text (self);
 	case MU_MSG_FIELD_ID_BODY_HTML: *do_free = TRUE;
 		return get_body (self, TRUE);
 
-	case MU_MSG_FIELD_ID_CC: *do_free = TRUE;
-		return get_recipient (self, GMIME_RECIPIENT_TYPE_CC);
-
 	case MU_MSG_FIELD_ID_FROM:
 		return (char*)maybe_cleanup
 			(g_mime_message_get_sender (self->_mime_msg),
 			 self->_path, do_free);
 
 	case MU_MSG_FIELD_ID_PATH: return self->_path;
-		
+
 	case MU_MSG_FIELD_ID_SUBJECT:
 		return (char*)maybe_cleanup
 			(g_mime_message_get_subject (self->_mime_msg),
 			 self->_path, do_free);
 
-	case MU_MSG_FIELD_ID_TO: *do_free = TRUE;
-		return get_recipient (self, GMIME_RECIPIENT_TYPE_TO);
-
 	case MU_MSG_FIELD_ID_MSGID:
 		return (char*)g_mime_message_get_message_id (self->_mime_msg);
 
 	case MU_MSG_FIELD_ID_MAILDIR: return self->_maildir;
-		
+
 	default: g_return_val_if_reached (NULL);
 	}
 }
@@ -732,9 +820,9 @@ mu_msg_file_get_str_list_field (MuMsgFile *self, MuMsgFieldId mfid,
 {
 	g_return_val_if_fail (self, NULL);
 	g_return_val_if_fail (mu_msg_field_is_string_list(mfid), NULL);
-	
+
 	switch (mfid) {
-		
+
 	case MU_MSG_FIELD_ID_REFS:
 		*do_free = TRUE;
 		return get_references (self);
@@ -752,15 +840,15 @@ mu_msg_file_get_num_field (MuMsgFile *self, const MuMsgFieldId mfid)
 {
 	g_return_val_if_fail (self, -1);
 	g_return_val_if_fail (mu_msg_field_is_numeric(mfid), -1);
-	
+
 	switch (mfid) {
-		
+
 	case MU_MSG_FIELD_ID_DATE: {
 		time_t t;
 		g_mime_message_get_date (self->_mime_msg, &t, NULL);
 		return (time_t)t;
 	}
-		
+
 	case MU_MSG_FIELD_ID_FLAGS:
 		return (gint64)get_flags(self);
 
@@ -779,9 +867,16 @@ mu_msg_file_get_num_field (MuMsgFile *self, const MuMsgFieldId mfid)
 const char*
 mu_msg_file_get_header (MuMsgFile *self, const char *header)
 {
+	const gchar *hdr;
+
 	g_return_val_if_fail (self, NULL);
 	g_return_val_if_fail (header, NULL);
-	
-	return g_mime_object_get_header
-		(GMIME_OBJECT(self->_mime_msg), header);
+
+	/* sadly, g_mime_object_get_header may return non-ascii;
+	 * so, we need to ensure that
+	 */
+	hdr = g_mime_object_get_header (GMIME_OBJECT(self->_mime_msg),
+					header);
+
+	return hdr ? free_string_later (self, mu_str_utf8ify(hdr)) : NULL;
 }
