@@ -6,16 +6,16 @@
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation; either version 3 of the License, or
 ** (at your option) any later version.
-**  
+**
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
-**  
+**
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software Foundation,
-** Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
-**  
+** Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+**
 */
 
 #include <string.h>
@@ -34,17 +34,20 @@
 #include "mu-util.h"
 #include "mu-str.h"
 
+#include "mu-maildir.h"
 
 /* note, we do the gmime initialization here rather than in
  * mu-runtime, because this way we don't need mu-runtime for simple
- * cases -- such as our unit tests */
+ * cases -- such as our unit tests. Also note that we need gmime init
+ * even for the doc backend, as we use the address parsing functions
+ * also there. */
 static gboolean _gmime_initialized = FALSE;
 
 static void
 gmime_init (void)
 {
 	g_return_if_fail (!_gmime_initialized);
-	
+
 #ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
 	g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
 #else
@@ -57,14 +60,13 @@ gmime_init (void)
 static void
 gmime_uninit (void)
 {
-		g_return_if_fail (_gmime_initialized);
+	g_return_if_fail (_gmime_initialized);
 
-		g_mime_shutdown();
-		_gmime_initialized = FALSE;
+	g_mime_shutdown();
+	_gmime_initialized = FALSE;
 }
 
 
-
 static MuMsg*
 msg_new (void)
 {
@@ -74,7 +76,10 @@ msg_new (void)
 
 	self->_refcount = 1;
 	self->_cache = mu_msg_cache_new ();
- 
+
+	self->_file = NULL;
+	self->_doc  = NULL;
+
 	return self;
 }
 
@@ -83,31 +88,41 @@ mu_msg_new_from_file (const char *path, const char *mdir, GError **err)
 {
 	MuMsg *self;
 	MuMsgFile *msgfile;
-	
+
 	g_return_val_if_fail (path, NULL);
 
 	if (G_UNLIKELY(!_gmime_initialized)) {
 		gmime_init ();
 		g_atexit (gmime_uninit);
 	}
-		
+
 	msgfile = mu_msg_file_new (path, mdir, err);
-	if (!msgfile) 
+	if (!msgfile)
 		return NULL;
-	
+
 	self = msg_new ();
 	self->_file = msgfile;
-		
+
 	return self;
 }
 
 
+void
+mu_msg_close_file_backend (MuMsg *msg)
+{
+	g_return_if_fail (msg);
+
+	mu_msg_file_destroy (msg->_file);
+	msg->_file = NULL;
+}
+
+
 MuMsg*
 mu_msg_new_from_doc (XapianDocument *doc, GError **err)
 {
 	MuMsg *self;
 	MuMsgDoc *msgdoc;
-		
+
 	g_return_val_if_fail (doc, NULL);
 
 	if (G_UNLIKELY(!_gmime_initialized)) {
@@ -121,22 +136,22 @@ mu_msg_new_from_doc (XapianDocument *doc, GError **err)
 
 	self = msg_new ();
 	self->_doc = msgdoc;
-			
+
 	return self;
 }
 
 
-static void 
+static void
 mu_msg_destroy (MuMsg *self)
 {
 	if (!self)
 		return;
 
 	mu_msg_file_destroy (self->_file);
-	mu_msg_doc_destroy (self->_doc);
+	mu_msg_doc_destroy  (self->_doc);
 
 	mu_msg_cache_destroy (self->_cache);
-		
+
 	g_slice_free (MuMsg, self);
 }
 
@@ -147,7 +162,7 @@ mu_msg_ref (MuMsg *self)
 	g_return_val_if_fail (self, NULL);
 
 	++self->_refcount;
-	
+
 	return self;
 }
 
@@ -156,8 +171,8 @@ mu_msg_unref (MuMsg *self)
 {
 	g_return_if_fail (self);
 	g_return_if_fail (self->_refcount >= 1);
-	
-	if (--self->_refcount == 0) 
+
+	if (--self->_refcount == 0)
 		mu_msg_destroy (self);
 }
 
@@ -170,7 +185,7 @@ get_path (MuMsg *self)
 	const char *path;
 	char *val;
 	gboolean do_free;
-		
+
 	/* try to get the path from the cache */
 	path = mu_msg_cache_str (self->_cache, MU_MSG_FIELD_ID_PATH);
 	if (path)
@@ -183,14 +198,14 @@ get_path (MuMsg *self)
 		val = mu_msg_doc_get_str_field (self->_doc,
 						MU_MSG_FIELD_ID_PATH,
 						&do_free);
-		
+
 	/* not in the cache yet? try to get it from the file backend,
 	 * in case we are using that */
 	if (!val && self->_file)
 		val = mu_msg_file_get_str_field (self->_file,
 						 MU_MSG_FIELD_ID_PATH,
 						 &do_free);
-		
+
 	/* this cannot happen unless there are bugs in mu */
 	if (!val) {
 		g_warning ("%s: cannot find path", __FUNCTION__);
@@ -214,7 +229,7 @@ get_msg_file (MuMsg *self)
 
 	if (!(path = get_path (self)))
 		return NULL;
-		
+
 	err = NULL;
 	mfile = mu_msg_file_new (path, NULL, &err);
 	if (!mfile) {
@@ -223,7 +238,7 @@ get_msg_file (MuMsg *self)
 		g_error_free (err);
 		return NULL;
 	}
-		
+
 	return mfile;
 }
 
@@ -254,7 +269,7 @@ get_str_list_field (MuMsg *self, MuMsgFieldId mfid)
 		val = mu_msg_file_get_str_list_field (self->_file, mfid,
 						      &do_free);
 	}
-		
+
 	/* if we get a string that needs freeing, we tell the cache to
 	 * mark the string as such, so it will be freed when the cache
 	 * is freed (or when the value is overwritten) */
@@ -292,7 +307,7 @@ get_str_field (MuMsg *self, MuMsgFieldId mfid)
 		g_warning ("%s: cannot retrieve field", __FUNCTION__);
 		return NULL;
 	}
-		
+
 	/* if we get a string that needs freeing, we tell the cache to
 	 * mark the string as such, so it will be freed when the cache
 	 * is freed (or when the value is overwritten) */
@@ -304,10 +319,11 @@ static gint64
 get_num_field (MuMsg *self, MuMsgFieldId mfid)
 {
 	guint64 val;
-		 
+
+	/* first try the cache */
 	if (mu_msg_cache_cached (self->_cache, mfid))
 		return mu_msg_cache_num (self->_cache, mfid);
-		
+
 	/* if it's not in the cache but it is a value retrievable from
 	 * the doc backend, use that */
 	val = -1;
@@ -344,7 +360,7 @@ mu_msg_get_header (MuMsg *self, const char *header)
 }
 
 
-const char*    
+const char*
 mu_msg_get_path (MuMsg *self)
 {
 	g_return_val_if_fail (self, NULL);
@@ -352,21 +368,21 @@ mu_msg_get_path (MuMsg *self)
 }
 
 
-const char*    
+const char*
 mu_msg_get_subject  (MuMsg *self)
 {
 	g_return_val_if_fail (self, NULL);
 	return get_str_field (self, MU_MSG_FIELD_ID_SUBJECT);
 }
 
-const char*    
+const char*
 mu_msg_get_msgid  (MuMsg *self)
 {
 	g_return_val_if_fail (self, NULL);
 	return get_str_field (self, MU_MSG_FIELD_ID_MSGID);
 }
 
-const char*    
+const char*
 mu_msg_get_maildir (MuMsg *self)
 {
 	g_return_val_if_fail (self, NULL);
@@ -374,7 +390,7 @@ mu_msg_get_maildir (MuMsg *self)
 }
 
 
-const char*    
+const char*
 mu_msg_get_from (MuMsg *self)
 {
 	g_return_val_if_fail (self, NULL);
@@ -382,14 +398,14 @@ mu_msg_get_from (MuMsg *self)
 }
 
 
-const char*    
+const char*
 mu_msg_get_to (MuMsg *self)
 {
 	g_return_val_if_fail (self, NULL);
 	return get_str_field (self, MU_MSG_FIELD_ID_TO);
 }
 
-const char*    
+const char*
 mu_msg_get_cc (MuMsg *self)
 {
 	g_return_val_if_fail (self, NULL);
@@ -397,7 +413,7 @@ mu_msg_get_cc (MuMsg *self)
 }
 
 
-const char*    
+const char*
 mu_msg_get_bcc (MuMsg *self)
 {
 	g_return_val_if_fail (self, NULL);
@@ -414,11 +430,11 @@ mu_msg_get_date (MuMsg *self)
 
 
 
-MuMsgFlags
+MuFlags
 mu_msg_get_flags (MuMsg *self)
 {
-	g_return_val_if_fail (self, MU_MSG_FLAG_NONE);
-	return (MuMsgFlags)get_num_field (self, MU_MSG_FIELD_ID_FLAGS);
+	g_return_val_if_fail (self, MU_FLAG_NONE);
+	return (MuFlags)get_num_field (self, MU_MSG_FIELD_ID_FLAGS);
 }
 
 size_t
@@ -436,13 +452,6 @@ mu_msg_get_prio (MuMsg *self)
 	return (MuMsgPrio)get_num_field (self, MU_MSG_FIELD_ID_PRIO);
 }
 
-time_t
-mu_msg_get_timestamp (MuMsg *self)
-{
-	g_return_val_if_fail (self, (time_t)-1);
-	return (MuMsgPrio)get_num_field (self, MU_MSG_FIELD_ID_TIMESTAMP);
-}
-
 
 const char*
 mu_msg_get_body_html (MuMsg *self)
@@ -507,7 +516,7 @@ mu_msg_contact_new (const char *name, const char *address,
 		    MuMsgContactType type)
 {
 	MuMsgContact *self;
-	
+
 	g_return_val_if_fail (name, NULL);
 	g_return_val_if_fail (address, NULL);
 	g_return_val_if_fail (!mu_msg_contact_type_is_valid(type),
@@ -519,7 +528,7 @@ mu_msg_contact_new (const char *name, const char *address,
 	self->address = g_strdup (address);
 	self->type    = type;
 
-	return self;	
+	return self;
 }
 
 
@@ -534,7 +543,7 @@ mu_msg_contact_destroy (MuMsgContact *self)
 	g_free ((void*)self->address);
 	g_slice_free (MuMsgContact, self);
 }
-	
+
 
 static gboolean
 fill_contact (MuMsgContact *self, InternetAddress *addr,
@@ -542,10 +551,10 @@ fill_contact (MuMsgContact *self, InternetAddress *addr,
 {
 	if (!addr)
 		return FALSE;
-	
+
 	self->name = internet_address_get_name (addr);
-	self->type = ctype;  
-	
+	self->type = ctype;
+
 	/* we only support internet mailbox addresses; if we don't
 	 * check, g_mime hits an assert
 	 */
@@ -554,7 +563,7 @@ fill_contact (MuMsgContact *self, InternetAddress *addr,
 			(INTERNET_ADDRESS_MAILBOX(addr));
 	else
 		self->address  = NULL;
-	
+
 	return TRUE;
 }
 
@@ -564,16 +573,16 @@ address_list_foreach (InternetAddressList *addrlist, MuMsgContactType ctype,
 		      MuMsgContactForeachFunc func, gpointer user_data)
 {
 	int i;
-		
+
 	for (i = 0; addrlist && i != internet_address_list_length(addrlist);
 	     ++i) {
-		
+
 		MuMsgContact contact;
 		if (!fill_contact(&contact,
 				  internet_address_list_get_address (addrlist, i),
 				  ctype))
 			continue;
-		
+
 		if (!(func)(&contact, user_data))
 			break;
 	}
@@ -588,7 +597,7 @@ addresses_foreach (const char* addrs, MuMsgContactType ctype,
 
 	if (!addrs)
 		return;
-	
+
 	addrlist = internet_address_list_parse_string (addrs);
 	if (addrlist) {
 		address_list_foreach (addrlist, ctype, func, user_data);
@@ -598,11 +607,11 @@ addresses_foreach (const char* addrs, MuMsgContactType ctype,
 
 
 void
-msg_contact_foreach_file (MuMsg *msg, MuMsgContactForeachFunc func, 
+msg_contact_foreach_file (MuMsg *msg, MuMsgContactForeachFunc func,
 			gpointer user_data)
 {
-	int i;		
-	struct { 
+	int i;
+	struct {
 		GMimeRecipientType     _gmime_type;
 		MuMsgContactType       _type;
 	} ctypes[] = {
@@ -614,7 +623,7 @@ msg_contact_foreach_file (MuMsg *msg, MuMsgContactForeachFunc func,
 	/* sender */
 	addresses_foreach (g_mime_message_get_sender (msg->_file->_mime_msg),
 			   MU_MSG_CONTACT_TYPE_FROM, func, user_data);
-	
+
 	/* get to, cc, bcc */
 	for (i = 0; i != G_N_ELEMENTS(ctypes); ++i) {
 		InternetAddressList *addrlist;
@@ -626,7 +635,7 @@ msg_contact_foreach_file (MuMsg *msg, MuMsgContactForeachFunc func,
 
 
 static void
-msg_contact_foreach_doc (MuMsg *msg, MuMsgContactForeachFunc func, 
+msg_contact_foreach_doc (MuMsg *msg, MuMsgContactForeachFunc func,
 			 gpointer user_data)
 {
 	addresses_foreach (mu_msg_get_from (msg),
@@ -641,7 +650,7 @@ msg_contact_foreach_doc (MuMsg *msg, MuMsgContactForeachFunc func,
 
 
 void
-mu_msg_contact_foreach (MuMsg *msg, MuMsgContactForeachFunc func, 
+mu_msg_contact_foreach (MuMsg *msg, MuMsgContactForeachFunc func,
 			gpointer user_data)
 {
 	g_return_if_fail (msg);
@@ -666,7 +675,7 @@ cmp_str (const char* s1, const char *s2)
 		return -1;
 	else if (!s2)
 		return 1;
-	
+
 	return g_utf8_collate (s1, s2);
 }
 
@@ -680,7 +689,7 @@ cmp_subject (const char* s1, const char *s2)
 		return -1;
 	else if (!s2)
 		return 1;
-	
+
 	return g_utf8_collate (
 		mu_str_subject_normalize (s1),
 		mu_str_subject_normalize (s2));
@@ -693,162 +702,117 @@ mu_msg_cmp (MuMsg *m1, MuMsg *m2, MuMsgFieldId mfid)
 	g_return_val_if_fail (m1, 0);
 	g_return_val_if_fail (m2, 0);
 	g_return_val_if_fail (mu_msg_field_id_is_valid(mfid), 0);
-	
-	if (mfid == MU_MSG_FIELD_ID_SUBJECT) 
+
+	if (mfid == MU_MSG_FIELD_ID_SUBJECT)
 		return cmp_subject (get_str_field (m1, mfid),
 				    get_str_field (m2, mfid));
-	
-	if (mu_msg_field_is_string (mfid)) 
+
+	if (mu_msg_field_is_string (mfid))
 		return cmp_str (get_str_field (m1, mfid),
 				get_str_field (m2, mfid));
 
 	/* TODO: note, we cast (potentially > MAXINT to int) */
-	if (mu_msg_field_is_numeric (mfid)) 
+	if (mu_msg_field_is_numeric (mfid))
 		return get_num_field(m1, mfid) - get_num_field(m2, mfid);
-	
+
 	return 0; /* TODO: handle lists */
 }
 
 
-
-
-enum _MaildirType {
-	MAILDIR_TYPE_CUR,
-	MAILDIR_TYPE_NEW,
-	MAILDIR_TYPE_OTHER
-};
-typedef enum _MaildirType MaildirType;
-
-static MaildirType
-get_maildir_type (const char *path)
+gboolean
+mu_msg_is_readable (MuMsg *self)
 {
-	MaildirType mtype;
-	gchar *dirname;
-	
-	dirname = g_path_get_dirname (path);
-	/* g_path_get_dirname does not specify if the name includes
-	 * the closing '/'... if it does, remove it */
-	if (dirname[strlen(dirname) - 1 ] == G_DIR_SEPARATOR)
-		dirname[strlen(dirname) - 1] = '\0'; 
+	g_return_val_if_fail (self, FALSE);
 
-	if (g_str_has_suffix (dirname, "cur"))
-		mtype = MAILDIR_TYPE_CUR;
-	else if (g_str_has_suffix (dirname, "new"))
-		mtype = MAILDIR_TYPE_CUR;
-	else
-		mtype = MAILDIR_TYPE_OTHER;
-
-	g_free (dirname);
-
-	return mtype;
+	return (access (get_str_field (self, MU_MSG_FIELD_ID_PATH), R_OK)
+		== 0) ? TRUE : FALSE;
 }
 
 
+
+/* we need do to determine the
+ *   /home/foo/Maildir/bar
+ * from the /bar
+ * that we got
+ */
 char*
-get_new_fullpath (const char *oldpath, const char *targetmdir,
-		  MaildirType mtype)
+get_target_mdir (MuMsg *msg, const char *maildir, GError **err)
 {
-	char *filename, *newfullpath;
-	const char* mdirsub;
-	
-	filename = g_path_get_basename (oldpath);
-	
-	if (mtype == MAILDIR_TYPE_CUR)
-		mdirsub = "cur";
-	else if (mtype == MAILDIR_TYPE_NEW)
-		mdirsub = "new";
-	else {
-		g_free (filename);
-		g_return_val_if_reached (NULL);
+	char *rootmaildir, *rv;
+
+	if (!mu_msg_get_maildir(msg)) {
+		g_set_error (err, 0, MU_ERROR_GMIME,
+			     "message misses maildir "
+			     "field, so cannot be moved");
 		return NULL;
 	}
 
-	newfullpath = g_strdup_printf ("%s%c%s%c%s",
-				       targetmdir,
-				       G_DIR_SEPARATOR,
-				       mdirsub,
-				       G_DIR_SEPARATOR,
-				       filename);
-	g_free (filename);
-	return newfullpath;
-	
+	rootmaildir = mu_maildir_get_maildir_from_path (mu_msg_get_path(msg));
+	if (!rootmaildir)
+		return NULL;
+
+	if (!g_str_has_suffix (rootmaildir, mu_msg_get_maildir(msg))) {
+		g_set_error (err, 0, MU_ERROR_FILE,
+			     "path is %s, but maildir is %s",
+			     mu_msg_get_path(msg), mu_msg_get_maildir(msg));
+		g_free (rootmaildir);
+		return NULL;
+	}
+
+	rootmaildir[strlen(rootmaildir) - strlen (mu_msg_get_maildir(msg))] = '\0';
+	rv = g_strconcat (rootmaildir, maildir, NULL);
+	g_free (rootmaildir);
+
+	return rv;
 }
 
 
-static gboolean
-msg_move (const char* oldpath, const char *newfullpath, GError **err)
-{
-	if (access (oldpath, R_OK) != 0) {
-		g_set_error (err, 0, MU_ERROR_FILE, "cannot read %s",
-			     oldpath);
-		return FALSE;
-	}
 
 
-	if (access (newfullpath, F_OK) == 0) {
-		g_set_error (err, 0, MU_ERROR_FILE, "%s already exists",
-			     newfullpath);
-		return FALSE;
-	}
-	
-	if (rename (oldpath, newfullpath) != 0) {
-		g_set_error (err, 0, MU_ERROR_FILE, "error moving %s to %s",
-			     oldpath, newfullpath);
-		return FALSE;
-	}
-
-	/* double check -- is the target really there? */
-	if (access (newfullpath, F_OK) != 0) {
-		g_set_error (err, 0, MU_ERROR_FILE, "can't find target (%s)",
-			     newfullpath);
-		return FALSE;
-	}
-	
-	if (access (oldpath, F_OK) == 0) {
-		g_set_error (err, 0, MU_ERROR_FILE, "source is still there (%s)",
-			     oldpath);
-		return FALSE;
-	}
-	
-	return TRUE;
-}
-
 /*
  * move a msg to another maildir, trying to maintain 'integrity',
  * ie. msg in 'new/' will go to new/, one in cur/ goes to cur/. be
  * super-paranoid here...
  */
 gboolean
-mu_msg_move_to_maildir (MuMsg *self, const char* targetmdir, GError **err)
+mu_msg_move_to_maildir (MuMsg *self, const char *maildir,
+			MuFlags flags, gboolean ignore_dups, GError **err)
 {
-	const char *oldpath;
-	MaildirType mtype;
 	char *newfullpath;
-	
+	char *targetmdir;
+
 	g_return_val_if_fail (self, FALSE);
-	g_return_val_if_fail (targetmdir, FALSE);
-	g_return_val_if_fail (g_path_is_absolute(targetmdir), FALSE);
-	g_return_val_if_fail (mu_util_check_dir (targetmdir, TRUE, TRUE), FALSE);
+	g_return_val_if_fail (maildir, FALSE);     /* i.e. "/inbox" */
 
-	oldpath = mu_msg_get_path (self);
-	
-	mtype = get_maildir_type (oldpath);
-	g_return_val_if_fail (mtype==MAILDIR_TYPE_CUR||mtype==MAILDIR_TYPE_NEW,
-			  FALSE);	
-	
-	newfullpath = get_new_fullpath (oldpath, targetmdir, mtype);
-	g_return_val_if_fail (newfullpath, FALSE);
+	targetmdir = get_target_mdir (self, maildir, err);
+	if (!targetmdir)
+		return FALSE;
 
-	if (!msg_move (oldpath, newfullpath, err))
-		goto error;
-	
-	/* update our path to new one... */
-	mu_msg_cache_set_str (self->_cache, MU_MSG_FIELD_ID_PATH, newfullpath,
-			      TRUE);
-	return TRUE;
-	
-error:
-	g_free (newfullpath);
-	return FALSE;
+	newfullpath = mu_maildir_move_message (mu_msg_get_path (self),
+					       targetmdir, flags,
+					       ignore_dups, err);
+	g_free (targetmdir);
 
+	/* update the message path and the flags; they may have
+	 * changed */
+	if (newfullpath) {
+		mu_msg_cache_set_str (self->_cache, MU_MSG_FIELD_ID_PATH, newfullpath,
+				      TRUE); /* the cache will free the string */
+		mu_msg_cache_set_str (self->_cache, MU_MSG_FIELD_ID_MAILDIR,
+				      g_strdup(maildir), TRUE);
+		/* the cache will free the string */
+
+		/* the contentflags haven't changed, so make sure they persist */
+		flags |= mu_msg_get_flags (self) &
+			(MU_FLAG_HAS_ATTACH|MU_FLAG_ENCRYPTED|MU_FLAG_SIGNED);
+		/* update the pseudo-flag as well */
+		if (!(flags & MU_FLAG_NEW) && (flags & MU_FLAG_SEEN))
+			flags &= ~MU_FLAG_UNREAD;
+		else
+			flags |= MU_FLAG_UNREAD;
+
+		mu_msg_cache_set_num (self->_cache, MU_MSG_FIELD_ID_FLAGS, flags);
+	}
+
+	return newfullpath ? TRUE : FALSE;
 }
