@@ -1,6 +1,6 @@
 /* -*- mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
-** 
-** Copyright (C) 2008-2011 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+**
+** Copyright (C) 2008-2012 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -14,8 +14,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software Foundation,
-** Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
-**  
+** Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+**
 */
 
 #if HAVE_CONFIG_H
@@ -38,16 +38,16 @@ static gchar*
 fill_contacts_cache (void)
 {
 	gchar *cmdline, *tmpdir;
-	
+
 	tmpdir = test_mu_common_get_random_tmpdir();
 	cmdline = g_strdup_printf ("%s index --muhome=%s --maildir=%s"
 				   " --quiet",
 				   MU_PROGRAM, tmpdir, MU_TESTMAILDIR);
-	
+
 	g_assert (g_spawn_command_line_sync (cmdline, NULL, NULL,
 					     NULL, NULL));
 	g_free (cmdline);
-		
+
 	return tmpdir;
 }
 
@@ -56,14 +56,14 @@ static void
 test_mu_cfind_plain (void)
 {
 	gchar *muhome, *cmdline, *output, *erroutput;
-	
+
 	muhome = fill_contacts_cache ();
 	g_assert (muhome != NULL);
 
 	cmdline = g_strdup_printf ("%s cfind --muhome=%s --format=plain "
 				   "'testmu\\.xxx?'",
 				   MU_PROGRAM, muhome);
-	
+
 	output = erroutput = NULL;
 	g_assert (g_spawn_command_line_sync (cmdline, &output, &erroutput,
 					     NULL, NULL));
@@ -86,20 +86,20 @@ test_mu_cfind_bbdb (void)
 	struct tm *tmtoday;
 	time_t now;
 	const char *old_tz;
-	
+
 	muhome = fill_contacts_cache ();
 	g_assert (muhome != NULL);
-	
+
 	old_tz = set_tz ("Europe/Helsinki");
-	
+
 	cmdline = g_strdup_printf ("%s cfind --muhome=%s --format=bbdb "
 				   "'testmu\\.xxx?'",
 				   MU_PROGRAM, muhome);
-	
+
 	output = erroutput = NULL;
 	g_assert (g_spawn_command_line_sync (cmdline, &output, &erroutput,
 					     NULL, NULL));
-	
+
 	frm =   ";; -*-coding: utf-8-emacs;-*-\n"
 		";;; file-version: 6\n"
 		"[\"Helmut\" \"Kröger\" nil nil nil nil (\"hk@testmu.xxx\") "
@@ -112,10 +112,10 @@ test_mu_cfind_bbdb (void)
 	now = time(NULL);
 	tmtoday = localtime(&now);
 	strftime(today,sizeof(today),"%Y-%m-%d", tmtoday);
-	expected = g_strdup_printf (frm, today, today);	
-	
+	expected = g_strdup_printf (frm, today, today);
+
 	/* g_print ("\n%s\n", output); */
-	
+
 	g_assert_cmpstr (output, ==, expected);
 
 	g_free (cmdline);
@@ -132,14 +132,14 @@ static void
 test_mu_cfind_wl (void)
 {
 	gchar *muhome, *cmdline, *output, *erroutput;
-	
+
 	muhome = fill_contacts_cache ();
 	g_assert (muhome != NULL);
 
 	cmdline = g_strdup_printf ("%s cfind --muhome=%s --format=wl "
 				   "'testmu\\.xxx?'",
 				   MU_PROGRAM, muhome);
-	
+
 	output = erroutput = NULL;
 	g_assert (g_spawn_command_line_sync (cmdline, &output, &erroutput,
 					     NULL, NULL));
@@ -158,14 +158,14 @@ static void
 test_mu_cfind_mutt_alias (void)
 {
 	gchar *muhome, *cmdline, *output, *erroutput;
-	
+
 	muhome = fill_contacts_cache ();
 	g_assert (muhome != NULL);
 
 	cmdline = g_strdup_printf ("%s cfind --muhome=%s --format=mutt-alias "
 				   "'testmu\\.xxx?'",
 				   MU_PROGRAM, muhome);
-	
+
 	output = erroutput = NULL;
 	g_assert (g_spawn_command_line_sync (cmdline, &output, &erroutput,
 					     NULL, NULL));
@@ -183,21 +183,21 @@ static void
 test_mu_cfind_mutt_ab (void)
 {
 	gchar *muhome, *cmdline, *output, *erroutput;
-	
+
 	muhome = fill_contacts_cache ();
 	g_assert (muhome != NULL);
 
 	cmdline = g_strdup_printf ("%s cfind --muhome=%s --format=mutt-ab "
 				   "'testmu\\.xxx?'",
 				   MU_PROGRAM, muhome);
-	
+
 	output = erroutput = NULL;
 	g_assert (g_spawn_command_line_sync (cmdline, &output, &erroutput,
 					     NULL, NULL));
 	g_assert_cmpstr (output,
 			 ==,
 			 "Matching addresses in the mu database:\n"
-			 "hk@testmu.xxx\tHelmut Kröger\t\n"	
+			 "hk@testmu.xxx\tHelmut Kröger\t\n"
 			 "testmu@testmu.xx\tMü\t\n");
 	g_free (cmdline);
 	g_free (muhome);
@@ -210,14 +210,14 @@ static void
 test_mu_cfind_org_contact (void)
 {
 	gchar *muhome, *cmdline, *output, *erroutput;
-	
+
 	muhome = fill_contacts_cache ();
 	g_assert (muhome != NULL);
 
 	cmdline = g_strdup_printf ("%s cfind --muhome=%s --format=org-contact "
 				   "'testmu\\.xxx?'",
 				   MU_PROGRAM, muhome);
-	
+
 	output = erroutput = NULL;
 	g_assert (g_spawn_command_line_sync (cmdline, &output, &erroutput,
 					     NULL, NULL));
@@ -244,14 +244,14 @@ static void
 test_mu_cfind_csv (void)
 {
 	gchar *muhome, *cmdline, *output, *erroutput;
-	
+
 	muhome = fill_contacts_cache ();
 	g_assert (muhome != NULL);
 
 	cmdline = g_strdup_printf ("%s cfind --muhome=%s --format=csv "
 				   "'testmu\\.xxx?'",
 				   MU_PROGRAM, muhome);
-	
+
 	output = erroutput = NULL;
 	g_assert (g_spawn_command_line_sync (cmdline, &output, &erroutput,
 					     NULL, NULL));
@@ -272,8 +272,9 @@ main (int argc, char *argv[])
 	int rv;
 	g_test_init (&argc, &argv, NULL);
 
-	setenv ("LC_ALL", "en_US.utf8", 1);
-	
+	if (!set_en_us_utf8_locale())
+		return 0; /* don't error out... */
+
 	g_test_add_func ("/mu-cmd-cfind/test-mu-cfind-plain", test_mu_cfind_plain);
 	g_test_add_func ("/mu-cmd-cfind/test-mu-cfind-bbdb",  test_mu_cfind_bbdb);
 	g_test_add_func ("/mu-cmd-cfind/test-mu-cfind-wl",  test_mu_cfind_wl);
@@ -285,7 +286,7 @@ main (int argc, char *argv[])
 			 test_mu_cfind_org_contact);
 	g_test_add_func ("/mu-cmd-cfind/test-mu-cfind-csv",
 			 test_mu_cfind_csv);
-	
+
 	g_log_set_handler (NULL,
 			   G_LOG_LEVEL_MASK | G_LOG_LEVEL_WARNING|
 			   G_LOG_FLAG_FATAL| G_LOG_FLAG_RECURSION,
@@ -295,4 +296,3 @@ main (int argc, char *argv[])
 
 	return rv;
 }
-
